#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.
from datetime import datetime
from typing import List, Optional

from pydantic import ValidationError as PydanticValidationError
from rest_framework.exceptions import ValidationError

from haupt.common.exceptions import AccessNotAuthorized, AccessNotFound
from haupt.db.abstracts.runs import BaseRun
from haupt.db.managers.artifacts import set_artifacts
from polyaxon.constants.metadata import (
    META_COPY_ARTIFACTS,
    META_DESTINATION_IMAGE,
    META_IS_EXTERNAL,
    META_REWRITE_PATH,
    META_UPLOAD_ARTIFACTS,
)
from polyaxon.exceptions import PolyaxonCompilerError, PolyaxonSchemaError
from polyaxon.polyaxonfile import CompiledOperationSpecification
from polyaxon.polyflow import V1CompiledOperation, V1Init, V1Operation
from polyaxon.polypod.compiler import resolver
from polyaxon.polypod.compiler.lineage.artifacts_collector import (
    collect_lineage_artifacts_path,
)
from polyaxon.schemas import V1RunPending
from polyaxon.schemas.types import V1ArtifactsType


class CorePlatformResolver(resolver.BaseResolver):
    def resolve_params(self):
        self.params = self.run.params or {}

    def resolve_io(self):
        if self.compiled_operation.inputs:
            self.run.inputs = {
                io.name: io.value for io in self.compiled_operation.inputs
            }
        if self.compiled_operation.outputs:
            self.run.outputs = {
                io.name: io.value for io in self.compiled_operation.outputs
            }

    def _get_meta_artifacts_presets(self) -> List:
        if not self.run.meta_info or META_COPY_ARTIFACTS not in self.run.meta_info:
            return []

        artifacts = self.run.meta_info.pop(META_COPY_ARTIFACTS)
        artifacts = V1ArtifactsType.read(artifacts)

        def get_relative_to_run_artifacts(v: str):
            paths = v.split("/")[1:]
            paths = ["{{ globals.run_artifacts_path }}"] + paths
            return "/".join(paths)

        # Populate all paths
        if artifacts.dirs:
            artifacts.dirs = [
                [d, get_relative_to_run_artifacts(d)] for d in artifacts.dirs
            ]
        if artifacts.files:
            artifacts.files = [
                [d, get_relative_to_run_artifacts(d)] for d in artifacts.files
            ]
        init = V1Init.construct(artifacts=artifacts)
        return [{"runPatch": {"init": [init.to_dict()]}}]

    def _get_meta_destination_image(self) -> Optional[str]:
        meta_info = self.run.meta_info or {}
        return meta_info.get(META_DESTINATION_IMAGE)

    def resolve_presets(self):
        for preset in self._get_meta_artifacts_presets():
            self.compiled_operation = CompiledOperationSpecification.apply_preset(
                config=self.compiled_operation, preset=preset
            )

    def _persist_artifacts(self):
        if self.artifacts:
            set_artifacts(self.run, self.artifacts)

    def _resolve_artifacts_lineage_state(self):
        self.artifacts = self.artifacts or []
        # Check upload and add it as a lineage
        if self.run.meta_info and META_UPLOAD_ARTIFACTS in self.run.meta_info:
            artifacts_path = self.run.meta_info[META_UPLOAD_ARTIFACTS]
            if artifacts_path:
                upload_artifact = collect_lineage_artifacts_path(artifacts_path)
                self.artifacts.append(upload_artifact)
        self._persist_artifacts()

    def _check_approval(self):
        if self.compiled_operation.is_approved is False and self.run.pending is None:
            self.run.pending = V1RunPending.APPROVAL
            return True
        return False

    def persist_state(self):
        self.run.content = self.compiled_operation.to_json()
        if (
            self.compiled_operation.is_service_run
            and self.compiled_operation.run.rewrite_path
        ):
            self.run.meta_info[META_REWRITE_PATH] = True
        if (
            self.compiled_operation.is_service_run
            and self.compiled_operation.run.is_external
        ):
            self.run.meta_info[META_IS_EXTERNAL] = True
        update_fields = ["content", "inputs", "outputs", "meta_info"]
        if self._check_approval():
            update_fields.append("pending")
        self.run.save(update_fields=update_fields)
        self._resolve_artifacts_lineage_state()


def resolve(
    run: BaseRun,
    compiled_at: Optional[datetime] = None,
    eager: bool = False,
    resolver_cls=None,
):
    resolver_cls = resolver_cls or CorePlatformResolver
    try:
        compiled_operation = V1CompiledOperation.read(
            run.content
        )  # TODO: Use construct
        project = run.project
        return resolver.resolve(
            run=run,
            compiled_operation=compiled_operation,
            owner_name=project.owner.name,
            project_name=project.name,
            project_uuid=project.uuid.hex,
            run_uuid=run.uuid.hex,
            run_name=run.name,
            run_path=run.subpath,
            resolver_cls=resolver_cls,
            params=None,
            compiled_at=compiled_at,
            created_at=run.created_at,
            cloning_kind=run.cloning_kind,
            original_uuid=run.original.uuid.hex if run.original_id else None,
            is_independent=bool(run.pipeline_id),
            eager=eager,
        )
    except (
        AccessNotAuthorized,
        AccessNotFound,
    ) as e:
        raise PolyaxonCompilerError("Access Error: %s" % e) from e
    except (
        AccessNotAuthorized,
        AccessNotFound,
        PydanticValidationError,
        PolyaxonSchemaError,
        ValidationError,
    ) as e:
        raise PolyaxonCompilerError("Compilation Error: %s" % e) from e


def resolve_hooks(run: BaseRun, resolver_cls=None) -> List[V1Operation]:
    resolver_cls = resolver_cls or CorePlatformResolver
    try:
        compiled_operation = V1CompiledOperation.read(
            run.content
        )  # TODO: Use construct
        project = run.project
        return resolver.resolve_hooks(
            run=run,
            compiled_operation=compiled_operation,
            owner_name=project.owner.name,
            project_name=project.name,
            project_uuid=project.uuid.hex,
            run_uuid=run.uuid.hex,
            run_name=run.name,
            run_path=run.subpath,
            resolver_cls=resolver_cls,
            params=None,
            compiled_at=None,
            created_at=run.created_at,
            cloning_kind=run.cloning_kind,
            original_uuid=run.original.uuid.hex if run.original_id else None,
        )
    except (
        AccessNotAuthorized,
        AccessNotFound,
    ) as e:
        raise PolyaxonCompilerError("Access Error: %s" % e) from e
    except (
        AccessNotAuthorized,
        AccessNotFound,
        PydanticValidationError,
        PolyaxonSchemaError,
        ValidationError,
    ) as e:
        raise PolyaxonCompilerError("Compilation Error: %s" % e) from e



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.
from typing import Dict, Optional

from haupt.common.config_manager import ConfigManager


def set_celery(
    context, config: ConfigManager, routes: Dict, schedules: Optional[Dict] = None
):
    context["CELERY_TASK_TRACK_STARTED"] = config.get(
        "POLYAXON_CELERY_TASK_TRACK_STARTED", "bool", is_optional=True, default=True
    )

    context["CELERY_BROKER_POOL_LIMIT"] = config.get(
        "POLYAXON_CELERY_BROKER_POOL_LIMIT", "int", is_optional=True, default=100
    )

    context["CELERY_BROKER_BACKEND"] = config.broker_backend
    confirm_publish = config.get(
        "POLYAXON_CELERY_CONFIRM_PUBLISH", "bool", is_optional=True, default=True
    )
    context["CELERY_CONFIRM_PUBLISH"] = confirm_publish
    if config.is_rabbitmq_broker and confirm_publish:
        # see https://github.com/celery/celery/issues/5410 for details
        context["CELERY_BROKER_TRANSPORT_OPTIONS"] = {"confirm_publish": True}

    context["CELERY_BROKER_URL"] = config.get_broker_url()

    context["INTERNAL_EXCHANGE"] = config.get(
        "POLYAXON_INTERNAL_EXCHANGE", "str", is_optional=True, default="internal"
    )

    result_bucked = config.get(
        "POLYAXON_REDIS_CELERY_RESULT_BACKEND_URL",
        "str",
        is_optional=True,
    )
    if result_bucked:
        context["CELERY_RESULT_BACKEND"] = config.get_redis_url(
            "POLYAXON_REDIS_CELERY_RESULT_BACKEND_URL"
        )

    context["CELERY_WORKER_PREFETCH_MULTIPLIER"] = config.get(
        "POLYAXON_CELERY_WORKER_PREFETCH_MULTIPLIER",
        "int",
        is_optional=True,
        default=4,
    )

    eager_mode = config.get(
        "POLYAXON_CELERY_TASK_ALWAYS_EAGER", "bool", is_optional=True, default=False
    )
    context["CELERY_TASK_ALWAYS_EAGER"] = eager_mode
    if eager_mode:
        context["CELERY_BROKER_TRANSPORT"] = "memory"

    context["CELERY_ACCEPT_CONTENT"] = ["application/json"]
    context["CELERY_TASK_DEFAULT_PRIORITY"] = 10
    context["CELERY_TASK_SERIALIZER"] = "json"
    context["CELERY_RESULT_SERIALIZER"] = "json"
    context["CELERY_TASK_IGNORE_RESULT"] = True
    context["CELERY_TIMEZONE"] = config.timezone
    context["CELERY_HARD_TIME_LIMIT_DELAY"] = config.get(
        "POLYAXON_CELERY_HARD_TIME_LIMIT_DELAY",
        "int",
        is_optional=True,
        default=180,
    )

    context["CELERY_WORKER_MAX_TASKS_PER_CHILD"] = config.get(
        "POLYAXON_CELERY_WORKER_MAX_TASKS_PER_CHILD",
        "int",
        is_optional=True,
        default=100,
    )

    context["CELERY_WORKER_MAX_MEMORY_PER_CHILD"] = config.get(
        "POLYAXON_CELERY_WORKER_MAX_MEMORY_PER_CHILD",
        "int",
        is_optional=True,
        default=400000,
    )

    class Intervals:
        """All intervals are in seconds"""

        OPERATIONS_DEFAULT_RETRY_DELAY = config.get(
            "POLYAXON_INTERVALS_OPERATIONS_DEFAULT_RETRY_DELAY",
            "int",
            is_optional=True,
            default=60,
        )
        OPERATIONS_MAX_RETRY_DELAY = config.get(
            "POLYAXON_INTERVALS_OPERATIONS_MAX_RETRY_DELAY",
            "int",
            is_optional=True,
            default=60 * 60,
        )
        RUNS_SCHEDULER = config.get(
            "POLYAXON_INTERVALS_RUNS_SCHEDULER", "int", is_optional=True, default=30
        )

    context["Intervals"] = Intervals
    context["CELERY_TASK_ROUTES"] = routes
    if schedules:
        context["CELERY_BEAT_SCHEDULE"] = schedules



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

"""
ASGI config for deploy project.

It exposes the ASGI callable as a module-level variable named ``application``.

For more information on this file, see
https://docs.djangoproject.com/en/4.1/howto/deployment/asgi/
"""

import os

from django.core.asgi import get_asgi_application

from polyaxon.env_vars.keys import EV_KEYS_SERVICE
from polyaxon.services.values import PolyaxonServices

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "haupt.polyconf.settings")
os.environ.setdefault("ASGI_APPLICATION", "haupt.polyconf.asgi.streams.application")
os.environ[EV_KEYS_SERVICE] = PolyaxonServices.STREAMS
application = get_asgi_application()



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from django.db import transaction

from haupt.background.celeryp.app import app
from haupt.background.celeryp.polyaxon_task import HauptTask

app.Task = HauptTask  # Custom base class for logging


def send(task_name, kwargs=None, **options):
    options["ignore_result"] = options.get("ignore_result", True)
    return transaction.on_commit(
        lambda: app.send_task(task_name, kwargs=kwargs, **options)
    )
    # if "countdown" not in options:
    #     options["countdown"] = conf.get(SCHEDULER_GLOBAL_COUNTDOWN)
    # return app.send_task(task_name, kwargs=kwargs, **options)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from django.conf import settings

from haupt.common.service_interface import LazyServiceWrapper
from haupt.orchestration.executor.manager import event_manager
from haupt.orchestration.executor.service import ExecutorService


def get_executor_backend_path():
    return (
        settings.EXECUTOR_BACKEND
        or "haupt.orchestration.executor.service.ExecutorService"
    )


def get_executor_options():
    return {"workers_service": settings.WORKERS_SERVICE}


backend = LazyServiceWrapper(
    backend_base=ExecutorService,
    backend_path=get_executor_backend_path(),
    options=get_executor_options(),
)
backend.expose(locals())

subscribe = event_manager.subscribe



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from rest_framework import serializers

from haupt.common import conf
from haupt.common.options.registry.k8s import K8S_NAMESPACE


class SettingsMixin:
    settings = serializers.SerializerMethodField()

    def get_settings(self, obj):
        return {"namespace": conf.get(K8S_NAMESPACE)}



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from typing import Dict


class Result:
    INFO = "info"
    WARNING = "warning"
    ERROR = "error"
    SEVERITY_VALUES = {INFO, WARNING, ERROR}

    def __init__(self, message="Service is healthy", severity=INFO):
        self.message = message
        if severity not in self.SEVERITY_VALUES:
            raise ValueError("Health check Error not recognized `{}`".format(severity))
        self.severity = severity

    def __str__(self) -> str:
        return self.message

    @property
    def is_healthy(self) -> bool:
        return self.severity == self.INFO

    @property
    def is_warning(self) -> bool:
        return self.severity == self.WARNING

    @property
    def is_error(self) -> bool:
        return self.severity == self.ERROR

    def to_dict(self) -> Dict:
        return {
            "is_healthy": self.is_healthy,
            "message": self.message,
            "severity": self.severity,
        }



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt.common.options import option_namespaces, option_subjects
from haupt.common.options.option import Option, OptionScope, OptionStores

# Global Async Countdown
SCHEDULER_GLOBAL_COUNTDOWN = "{}_{}".format(
    option_namespaces.SCHEDULER,
    option_subjects.GLOBAL_COUNTDOWN,
)

OPTIONS = {
    SCHEDULER_GLOBAL_COUNTDOWN,
}


class SchedulerCountdown(Option):
    key = SCHEDULER_GLOBAL_COUNTDOWN
    scope = OptionScope.GLOBAL
    is_secret = False
    is_optional = True
    is_list = False
    typing = "int"
    store = OptionStores.SETTINGS
    default = 1
    options = None
    description = "Global count down for scheduler"



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from django.http import Http404, HttpRequest
from django.shortcuts import get_object_or_404

from haupt.apis.endpoints.project import ProjectResourceEndpoint
from haupt.common.apis.regex import (
    ARTIFACT_NAME_KEY,
    NAME_KEY,
    OWNER_NAME_KEY,
    PROJECT_NAME_KEY,
    RUN_UUID_KEY,
    UUID_KEY,
)
from haupt.db.queries import artifacts as artifacts_queries
from haupt.db.queries import runs as runs_queries


class RunEndpoint(ProjectResourceEndpoint):
    queryset = runs_queries.run_model.all.select_related("project")
    lookup_field = UUID_KEY
    lookup_url_kwarg = RUN_UUID_KEY
    CONTEXT_KEYS = (OWNER_NAME_KEY, PROJECT_NAME_KEY, RUN_UUID_KEY)
    CONTEXT_OBJECTS = ("run",)

    def enrich_queryset(self, queryset):
        return queryset.filter(project__name=self.project_name)

    def set_owner(self):
        self.project = self.run.project
        self._owner_id = self.project.owner_id

    def initialize_object_context(self, request: HttpRequest, *args, **kwargs):
        #  pylint:disable=attribute-defined-outside-init
        self.run = self.get_object()


class RunResourceListEndpoint(RunEndpoint):
    AUDITOR_EVENT_TYPES = None

    def get_object(self):
        if self._object:
            return self._object

        self._object = get_object_or_404(
            runs_queries.run_model.all.select_related("project"),
            uuid=self.run_uuid,
            project__name=self.project_name,
        )

        # May raise a permission denied
        self.check_object_permissions(self.request, self._object)

        return self._object

    def enrich_queryset(self, queryset):
        return queryset.filter(run=self.run)


class RunArtifactEndpoint(RunEndpoint):
    AUDITOR_EVENT_TYPES = None
    lookup_field = NAME_KEY
    lookup_url_kwarg = ARTIFACT_NAME_KEY
    CONTEXT_KEYS = (PROJECT_NAME_KEY, RUN_UUID_KEY, ARTIFACT_NAME_KEY)
    CONTEXT_OBJECTS = ("run_artifact",)

    def enrich_queryset(self, queryset):
        return queryset.filter(
            run__uuid=self.run_uuid,
            run__project__name=self.project_name,
        )

    def set_owner(self):
        self.project = self.run_artifact.run.project
        self._owner_id = self.project.owner_id

    def initialize_object_context(self, request: HttpRequest, *args, **kwargs):
        #  pylint:disable=attribute-defined-outside-init
        self.run_artifact = self.get_object()

    def get_object(self):
        queryset = self.filter_queryset(self.get_queryset())
        try:
            return queryset.get(artifact__name=self.artifact_name)
        except artifacts_queries.lineage_model.DoesNotExist:
            raise Http404(
                "No %s matches the given query." % self.queryset.model._meta.object_name
            )



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt.apis.endpoints.run import RunArtifactEndpoint, RunResourceListEndpoint
from haupt.apis.methods import run_lineage as methods
from haupt.apis.serializers.artifacts import (
    RunArtifactNameSerializer,
    RunArtifactSerializer,
)
from haupt.common.apis.filters import OrderingFilter, QueryFilter
from haupt.common.apis.paginator import LargeLimitOffsetPagination
from haupt.common.apis.regex import RUN_UUID_KEY
from haupt.common.endpoints.base import (
    CreateEndpoint,
    DestroyEndpoint,
    ListEndpoint,
    RetrieveEndpoint,
)
from haupt.common.events.registry.run import RUN_NEW_ARTIFACTS
from haupt.db.queries import artifacts as artifacts_queries
from haupt.db.query_managers.artifact import ArtifactQueryManager


class RunArtifactListView(RunResourceListEndpoint, ListEndpoint, CreateEndpoint):
    queryset = artifacts_queries.artifacts
    serializer_class = RunArtifactSerializer
    pagination_class = LargeLimitOffsetPagination
    AUDITOR_EVENT_TYPES = {
        "POST": RUN_NEW_ARTIFACTS,
    }
    AUDIT_PROJECT_RESOURCES = True
    PROJECT_RESOURCE_KEY = RUN_UUID_KEY
    AUDIT_EXTRA_KEYS = ("artifacts",)
    AUDIT_INSTANCE = True

    filter_backends = (QueryFilter, OrderingFilter)
    query_manager = ArtifactQueryManager
    check_alive = ArtifactQueryManager.CHECK_ALIVE
    ordering = ArtifactQueryManager.FIELDS_DEFAULT_ORDERING
    ordering_fields = ArtifactQueryManager.FIELDS_ORDERING
    ordering_proxy_fields = ArtifactQueryManager.FIELDS_ORDERING_PROXY

    def create(self, request, *args, **kwargs):
        return methods.create(view=self, request=request, *args, **kwargs)

    def get_serializer_context(self):
        context = super().get_serializer_context()
        context["run"] = self.run
        return context


class RunArtifactNameListView(RunResourceListEndpoint, ListEndpoint):
    queryset = artifacts_queries.artifacts_names
    serializer_class = RunArtifactNameSerializer
    pagination_class = LargeLimitOffsetPagination


class RunArtifactDetailView(RunArtifactEndpoint, RetrieveEndpoint, DestroyEndpoint):
    queryset = artifacts_queries.artifacts
    serializer_class = RunArtifactSerializer

    def get_serializer_context(self):
        context = super().get_serializer_context()
        context["run"] = self.run_artifact.run
        return context



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("db", "0001_initial"),
    ]

    operations = [
        migrations.RenameField(
            model_name="run",
            old_name="run_time",
            new_name="duration",
        ),
    ]



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.
import uuid

from django.contrib.auth import get_user_model

from haupt.common import conf
from haupt.common.options.registry.installation import ORGANIZATION_KEY
from haupt.db.abstracts.projects import Owner


def get_dummy_key():
    first_joined = get_user_model().objects.order_by("date_joined").first()
    if first_joined:
        key = uuid.uuid5(Owner.uuid, str(first_joined.date_joined.timestamp())).hex
    else:
        key = uuid.uuid4().hex
    conf.set(ORGANIZATION_KEY, key)
    return key



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.
from haupt.db.abstracts.getter import get_run_model

STATUS_UPDATE_COLUMNS_ONLY = [
    "id",
    "status",
    "status_conditions",
    "started_at",
    "updated_at",
    "finished_at",
    "duration",
    "wait_time",
    "meta_info",
]
STATUS_UPDATE_COLUMNS_DEFER = [
    "original",
    "cloning_kind",
    "description",
    "inputs",
    "outputs",
    "tags",
    "description",
    "live_state",
    "readme",
    "content",
    "is_managed",
    "pending",
]
DEFAULT_COLUMNS_DEFER = ["description", "readme", "content", "raw_content"]

run_model = get_run_model()

runs = run_model.objects



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt.background.celeryp.tasks import CoreSchedulerCeleryTasks
from haupt.common import workers
from haupt.common.checks import health_task


@workers.app.task(name=CoreSchedulerCeleryTasks.SCHEDULER_HEALTH, ignore_result=False)
def scheduler_health(x, y):
    return health_task.health_task(x, y)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from rest_framework.urlpatterns import format_suffix_patterns

from django.urls import re_path

from haupt.apis.runs import views
from haupt.common.apis.urls import runs

runs_urlpatterns = [
    re_path(runs.URLS_RUNS_DETAILS, views.RunDetailView.as_view()),
    re_path(runs.URLS_RUNS_RESTART, views.RunRestartView.as_view()),
    re_path(runs.URLS_RUNS_RESUME, views.RunResumeView.as_view()),
    re_path(runs.URLS_RUNS_COPY, views.RunCopyView.as_view()),
    re_path(runs.URLS_RUNS_STOP, views.RunStopView.as_view()),
    re_path(runs.URLS_RUNS_APPROVE, views.RunApproveView.as_view()),
    re_path(runs.URLS_RUNS_STATUSES, views.RunStatusListView.as_view()),
    re_path(runs.URLS_RUNS_NAMESPACE, views.RunNamespaceView.as_view()),
]

urlpatterns = format_suffix_patterns(runs_urlpatterns)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.


class CoreSchedulerCeleryTasks:
    """Scheduler celery tasks.

    N.B. make sure that the task name is not < 128.
    """

    SCHEDULER_HEALTH = "scheduler_health"

    RUNS_PREPARE = "runs_prepare"
    RUNS_START = "runs_start"
    RUNS_BUILT = "runs_built"
    RUNS_STOP = "runs_stop"
    RUNS_DELETE = "runs_delete"
    RUNS_SET_ARTIFACTS = "runs_set_artifacts"



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.
from polyaxon.k8s.async_manager import AsyncK8SManager


async def get_pods(
    k8s_manager: AsyncK8SManager,
    run_uuid: str,
):
    pods = await k8s_manager.list_pods(
        label_selector=k8s_manager.get_managed_by_polyaxon(run_uuid)
    )
    pods_list = {}
    for pod in pods or []:
        pods_list[
            pod.metadata.name
        ] = k8s_manager.api_client.sanitize_for_serialization(pod)
    return pods_list



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.
from typing import Optional


def write_to_conf_file(
    name: str, content: str, path: Optional[str] = None, ext: str = "conf"
):
    with open("{}/{}.{}".format(path or ".", name, ext), "w") as f:
        f.write(content)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from polyaxon.k8s.async_manager import AsyncK8SManager
from polyaxon.k8s.custom_resources import operation as op_crd


async def get_k8s_operation(k8s_manager: AsyncK8SManager, resource_name: str):
    return await k8s_manager.get_custom_object(
        name=resource_name,
        group=op_crd.GROUP,
        version=op_crd.API_VERSION,
        plural=op_crd.PLURAL,
    )



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from typing import Any, Dict, Optional

from haupt.common.options.option import Option


class BaseConfHandler:
    def get(self, option: Option, owners: Optional[Dict[str, int]] = None) -> Any:
        raise NotImplementedError

    def set(self, option: Option, value: Any, owners: Optional[Dict[str, int]] = None):
        raise NotImplementedError

    def delete(self, option: Option, owners: Optional[Dict[str, int]] = None) -> Any:
        raise NotImplementedError



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

ENABLED = "ENABLED"
URL = "URL"
GLOBAL_COUNTDOWN = "GLOBAL_COUNTDOWN"
DEFAULT_PREFIX = "DEFAULT_PREFIX"
TOKEN = "TOKEN"
CONTAINER = "CONTAINER"



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.
import logging

from typing import List

from polyaxon import settings
from polyaxon.agents.spawners.async_spawner import AsyncSpawner
from polyaxon.lifecycle import V1StatusCondition
from polyaxon.operations import get_notifier_operation
from polyaxon.polyaxonfile import OperationSpecification
from polyaxon.polypod import compiler
from polyaxon.polypod.compiler.converters import PLATFORM_CONVERTERS

logger = logging.getLogger("haupt.streams.notification")


async def notify_run(
    namespace: str,
    owner: str,
    project: str,
    run_uuid: str,
    run_name: str,
    condition: V1StatusCondition,
    connections: List[str],
):
    spawner = AsyncSpawner(namespace=namespace)
    await spawner.k8s_manager.setup()
    for connection in connections:
        connection_type = settings.AGENT_CONFIG.connections_by_names.get(connection)
        if not connection_type:
            logger.warning(
                "Could not create notification using connection {}, "
                "the connection was not found or not set correctly.".format(
                    connection_type
                )
            )
            continue

        operation = get_notifier_operation(
            connection=connection,
            backend=connection_type.kind,
            owner=owner,
            project=project,
            run_uuid=run_uuid,
            run_name=run_name,
            condition=condition.to_dict(),
        )
        compiled_operation = OperationSpecification.compile_operation(operation)
        resource = compiler.make(
            owner_name=owner,
            project_name=project,
            project_uuid=project,
            run_uuid=run_uuid,
            run_name=run_name,
            run_path=run_uuid,
            compiled_operation=compiled_operation,
            params=operation.params,
            converters=PLATFORM_CONVERTERS,
        )
        await spawner.create(
            run_uuid=run_uuid,
            run_kind=compiled_operation.get_run_kind(),
            resource=resource,
        )



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from django.conf import settings

from haupt.common.conf.option_manager import option_manager
from haupt.common.conf.option_service import OptionService
from haupt.common.service_interface import LazyServiceWrapper


def get_conf_backend_path():
    return settings.CONF_BACKEND or "haupt.common.conf.service.ConfService"


def get_conf_options():
    return {"check_ownership": settings.CONF_CHECK_OWNERSHIP}


backend = LazyServiceWrapper(
    backend_base=OptionService,
    backend_path=get_conf_backend_path(),
    options=get_conf_options(),
)
backend.expose(locals())

subscribe = option_manager.subscribe



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from django.db import migrations, models


def migrate_wait_time(apps, schema_editor):
    Run = apps.get_model("db", "Run")

    runs = []
    for r in Run.objects.annotate(
        calc_wait_time=models.F("started_at") - models.F("created_at")
    ):
        if r.calc_wait_time:
            r.wait_time = r.calc_wait_time.seconds
            runs.append(r)

    Run.objects.bulk_update(runs, ["wait_time"])


class Migration(migrations.Migration):
    dependencies = [
        ("db", "0007_auto_20201121_1332"),
    ]

    operations = [
        migrations.AddField(
            model_name="run",
            name="wait_time",
            field=models.IntegerField(blank=True, null=True),
        ),
        migrations.RunPython(migrate_wait_time),
    ]



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt import settings


def get_dns_config(dns_prefix=None, dns_backend=None, dns_cluster=None):
    dns_prefix = dns_prefix or settings.PROXIES_CONFIG.dns_prefix
    dns_backend = dns_backend or settings.PROXIES_CONFIG.dns_backend
    dns_cluster = dns_cluster or settings.PROXIES_CONFIG.dns_custom_cluster
    if not dns_prefix:
        dns_prefix = "{}.kube-system".format(dns_backend)
    return "{dns_prefix}.svc.{dns_cluster}".format(
        dns_prefix=dns_prefix, dns_cluster=dns_cluster
    )


def get_resolver():
    if settings.PROXIES_CONFIG.dns_use_resolver:
        dns_config = get_dns_config()
        return "resolver {} valid=5s;".format(dns_config)
    return ""



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from clipped.utils.enums import PEnum


class ContentTypes(str, PEnum):
    ORGANIZATION = "organization"
    TEAM = "team"
    USER = "user"
    PROJECT = "project"
    RUN = "run"



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt.common import conf
from haupt.common.options.registry import installation

conf.subscribe(installation.PlatformEnvironmentVersion)
conf.subscribe(installation.PlatformVersion)
conf.subscribe(installation.PlatformDist)
conf.subscribe(installation.PlatformHost)
conf.subscribe(installation.ChartVersion)
conf.subscribe(installation.OrganizationKey)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

import re

import django.core.validators

from django.db import migrations, models

import haupt.common.validation.blacklist


class Migration(migrations.Migration):
    dependencies = [
        ("db", "0006_auto_20201020_1705"),
    ]

    operations = [
        migrations.AddField(
            model_name="run",
            name="is_approved",
            field=models.BooleanField(
                default=True,
                help_text="If this entity requires approval before it should run.",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="name",
            field=models.CharField(
                max_length=128,
                validators=[
                    django.core.validators.RegexValidator(
                        re.compile("^[-a-zA-Z0-9_]+\\Z"),
                        "Enter a valid “slug” consisting of letters, numbers, underscores or hyphens.",
                        "invalid",
                    ),
                    haupt.common.validation.blacklist.validate_blacklist_name,
                ],
            ),
        ),
        migrations.AlterField(
            model_name="artifact",
            name="name",
            field=models.CharField(db_index=True, max_length=128),
        ),
        migrations.AlterField(
            model_name="run",
            name="name",
            field=models.CharField(
                blank=True,
                default=None,
                max_length=128,
                null=True,
                validators=[
                    django.core.validators.RegexValidator(
                        re.compile("^[-a-zA-Z0-9_]+\\Z"),
                        "Enter a valid “slug” consisting of letters, numbers, underscores or hyphens.",
                        "invalid",
                    ),
                ],
            ),
        ),
        migrations.AlterField(
            model_name="run",
            name="status",
            field=models.CharField(
                blank=True,
                choices=[
                    ("created", "created"),
                    ("resuming", "resuming"),
                    ("on_schedule", "on_schedule"),
                    ("compiled", "compiled"),
                    ("queued", "queued"),
                    ("scheduled", "scheduled"),
                    ("starting", "starting"),
                    ("running", "running"),
                    ("processing", "processing"),
                    ("stopping", "stopping"),
                    ("failed", "failed"),
                    ("stopped", "stopped"),
                    ("succeeded", "succeeded"),
                    ("skipped", "skipped"),
                    ("warning", "warning"),
                    ("unschedulable", "unschedulable"),
                    ("upstream_failed", "upstream_failed"),
                    ("retrying", "retrying"),
                    ("unknown", "unknown"),
                    ("done", "done"),
                ],
                db_index=True,
                default="created",
                max_length=16,
                null=True,
            ),
        ),
    ]



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from django.db import models

from polyaxon import live_state


class LiveManager(models.Manager):
    def get_queryset(self):
        qs = super().get_queryset()
        return qs.filter(live_state=live_state.STATE_LIVE)


class ArchivedManager(models.Manager):
    def get_queryset(self):
        qs = super().get_queryset()
        return qs.filter(live_state=live_state.STATE_ARCHIVED)


class RestorableManager(models.Manager):
    def get_queryset(self):
        qs = super().get_queryset()
        return qs.filter(
            live_state__in={live_state.STATE_LIVE, live_state.STATE_ARCHIVED}
        )



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from rest_framework import fields, serializers

from haupt.apis.serializers.base.is_managed import IsManagedMixin
from haupt.apis.serializers.base.project import ProjectMixin
from haupt.apis.serializers.project_resources import RunSerializer
from haupt.db.abstracts.getter import get_run_model


class RunStatusSerializer(serializers.ModelSerializer):
    uuid = fields.UUIDField(format="hex", read_only=True)
    condition = fields.DictField(
        write_only=True, allow_empty=True, allow_null=True, required=False
    )
    force = fields.BooleanField(write_only=True, required=False)

    class Meta:
        model = get_run_model()
        fields = (
            "uuid",
            "status",
            "condition",
            "status_conditions",
            "meta_info",
            "force",
        )
        extra_kwargs = {
            "status": {"read_only": True},
            "status_conditions": {"read_only": True},
        }


class RunDetailSerializer(RunSerializer, ProjectMixin, IsManagedMixin):
    project = fields.SerializerMethodField()
    merge = fields.BooleanField(write_only=True, required=False)

    class Meta(RunSerializer.Meta):
        fields = RunSerializer.Meta.fields + (
            "project",
            "readme",
            "description",
            "raw_content",
            "content",
            "live_state",
            "merge",
        )
        extra_kwargs = {
            "content": {"read_only": True},
            "raw_content": {"read_only": True},
        }

    def validate(self, attrs):
        attrs = super().validate(attrs)
        self.check_if_entity_is_managed(attrs=attrs, entity_name="Run")
        return attrs

    def validated_inputs(self, validated_data, inputs):
        new_inputs = validated_data.get("inputs")
        if not validated_data.get("merge") or not inputs or not new_inputs:
            # This is the default behavior
            return validated_data

        inputs.update(new_inputs)
        validated_data["inputs"] = inputs
        return validated_data

    def validated_meta(self, validated_data, meta_info):
        new_meta_info = validated_data.get("meta_info")
        if not validated_data.get("merge") or not meta_info or not new_meta_info:
            # This is the default behavior
            return validated_data

        meta_info.update(new_meta_info)
        validated_data["meta_info"] = meta_info
        return validated_data

    def validated_outputs(self, validated_data, outputs):
        new_outputs = validated_data.get("outputs")
        if not validated_data.get("merge") or not outputs or not new_outputs:
            # This is the default behavior
            return validated_data

        outputs.update(new_outputs)
        validated_data["outputs"] = outputs
        return validated_data

    def update(self, instance, validated_data):
        validated_data = self.validated_tags(
            validated_data=validated_data, tags=instance.tags
        )
        validated_data = self.validated_outputs(
            validated_data=validated_data, outputs=instance.outputs
        )
        validated_data = self.validated_inputs(
            validated_data=validated_data, inputs=instance.inputs
        )
        validated_data = self.validated_meta(
            validated_data=validated_data, meta_info=instance.meta_info
        )

        return super().update(instance=instance, validated_data=validated_data)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt.common.events.registry import run
from haupt.orchestration import executor

executor.subscribe(run.RunCreatedEvent)
executor.subscribe(run.RunResumedActorEvent)
executor.subscribe(run.RunStoppedActorEvent)
executor.subscribe(run.RunApprovedActorEvent)
executor.subscribe(run.RunNewStatusEvent)
executor.subscribe(run.RunDoneEvent)
executor.subscribe(run.RunDeletedActorEvent)
executor.subscribe(run.RunNewArtifactsEvent)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from celery import Celery, states

from django.apps import apps

STATES = states

app = Celery("haupt")

app.config_from_object("django.conf:settings", namespace="CELERY")
app.autodiscover_tasks(lambda: [n.name for n in apps.get_app_configs()])



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from rest_framework import fields, serializers


class ProjectMixin(serializers.Serializer):
    project = fields.SerializerMethodField()

    def get_project(self, obj):
        return obj.project.name



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from django.apps import AppConfig


class SchedulerConfig(AppConfig):
    name = "haupt.background.scheduler"
    verbose_name = "Scheduler"

    def ready(self):
        import haupt.db.signals.runs  # noqa
        from haupt.common import auditor, conf
        from haupt.orchestration import executor, operations

        conf.validate_and_setup()
        operations.validate_and_setup()
        executor.validate_and_setup()
        auditor.validate_and_setup()

        import haupt.common.options.conf_subscriptions  # noqa
        from haupt.common.events import auditor_subscriptions  # noqa



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt import pkg
from haupt.common import conf
from haupt.common.options.registry.core import (
    UI_ASSETS_VERSION,
    UI_BASE_URL,
    UI_ENABLED,
    UI_IN_SANDBOX,
    UI_OFFLINE,
)


def version(request):
    return {"version": pkg.VERSION}


def ui_base_url(request):
    return {"ui_base_url": conf.get(UI_BASE_URL)}


def ui_assets_version(request):
    return {
        "ui_assets_version": "{}.{}".format(pkg.VERSION, conf.get(UI_ASSETS_VERSION))
    }


def ui_offline(request):
    return {"ui_offline": conf.get(UI_OFFLINE)}


def ui_enabled(request):
    return {"ui_enabled": conf.get(UI_ENABLED)}


def ui_in_sandbox(request):
    return {"ui_in_sandbox": conf.get(UI_IN_SANDBOX)}



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.
from pydantic import ValidationError as PydanticValidationError
from rest_framework import fields, serializers
from rest_framework.exceptions import ValidationError

from django.db import IntegrityError

from haupt.apis.serializers.base.cloning import CloningMixin
from haupt.apis.serializers.base.is_managed import IsManagedMixin
from haupt.apis.serializers.base.pipeline import PipelineMixin
from haupt.apis.serializers.base.settings import SettingsMixin
from haupt.apis.serializers.base.tags import TagsMixin
from haupt.db.abstracts.getter import get_run_model
from haupt.db.managers.runs import create_run
from haupt.orchestration import operations
from polyaxon.exceptions import PolyaxonException
from polyaxon.polyaxonfile import OperationSpecification
from polyaxon.schemas import V1RunPending


class RunSerializer(
    serializers.ModelSerializer,
    CloningMixin,
    PipelineMixin,
    SettingsMixin,
    TagsMixin,
):
    uuid = fields.UUIDField(format="hex", read_only=True)
    original = fields.SerializerMethodField()
    pipeline = fields.SerializerMethodField()
    started_at = fields.DateTimeField(read_only=True)
    finished_at = fields.DateTimeField(read_only=True)
    settings = fields.SerializerMethodField()

    class Meta:
        model = get_run_model()
        fields = (
            "uuid",
            "name",
            "created_at",
            "updated_at",
            "started_at",
            "finished_at",
            "wait_time",
            "duration",
            "kind",
            "runtime",
            "meta_info",
            "status",
            "pipeline",
            "original",
            "is_managed",
            "pending",
            "inputs",
            "outputs",
            "tags",
            "settings",
        )
        extra_kwargs = {
            "is_managed": {"read_only": True},
        }


class OfflineRunSerializer(
    serializers.ModelSerializer,
    TagsMixin,
):
    uuid = fields.UUIDField(format="hex")
    created_at = fields.DateTimeField()

    class Meta:
        model = get_run_model()
        fields = (
            "uuid",
            "name",
            "description",
            "tags",
            "created_at",
            "updated_at",
            "started_at",
            "finished_at",
            "wait_time",
            "duration",
            "kind",
            "runtime",
            "meta_info",
            "status",
            "status_conditions",
            "is_managed",
            "inputs",
            "outputs",
            "content",
            "raw_content",
        )

    def create(self, validated_data, commit: bool = True):
        try:
            obj = super().create(validated_data)
        except IntegrityError:
            raise ValidationError(
                f"A run with uuid {validated_data.get('uuid')} already exists."
            )
        # Override auto-field for created_at
        created_at = validated_data.get("created_at")
        if created_at:
            obj.created_at = created_at
        if commit:
            obj.save()
        return obj


class OperationCreateSerializer(serializers.ModelSerializer, IsManagedMixin, TagsMixin):
    uuid = fields.UUIDField(format="hex", read_only=True)
    is_approved = fields.BooleanField(write_only=True, allow_null=True, required=False)

    class Meta:
        model = get_run_model()
        fields = (
            "uuid",
            "name",
            "description",
            "content",
            "is_managed",
            "pending",
            "meta_info",
            "tags",
            "is_approved",
        )
        extra_kwargs = {
            "is_approved": {"write_only": True},
        }

    def validate(self, attrs):
        attrs = super().validate(attrs)
        self.check_if_entity_is_managed(attrs=attrs, entity_name="Run")
        return attrs

    def create(self, validated_data):
        is_managed = validated_data["is_managed"]
        content = validated_data.get("content")
        meta_info = validated_data.get("meta_info") or {}
        if content:
            is_managed = True if is_managed is None else is_managed

        if is_managed and not content:
            raise ValidationError(
                "Managed runs require a content with valid specification"
            )

        project_id = validated_data["project"].id
        user = validated_data.get("user")
        name = validated_data.get("name")
        description = validated_data.get("description")
        tags = validated_data.get("tags")
        pending = validated_data.get("pending")
        # Check the deprecated `is_approved` flag
        if pending is None:
            is_approved = validated_data.get("is_approved")
            if is_approved is False:
                pending = V1RunPending.UPLOAD

        if is_managed or content:
            try:
                op_spec = OperationSpecification.read(content)
            except Exception as e:
                raise ValidationError(e)
            if op_spec.is_template():
                raise ValidationError(
                    "Received a template polyaxonfile, "
                    "Please customize the specification or disable the template."
                )
            try:
                return operations.init_and_save_run(
                    project_id=project_id,
                    user_id=user.id if user else None,
                    op_spec=op_spec,
                    name=name,
                    description=description,
                    tags=tags,
                    meta_info=meta_info,
                    is_managed=is_managed,
                    pending=pending,
                    supported_kinds=validated_data.get("supported_kinds"),
                    supported_owners=validated_data.get("supported_owners"),
                )
            except (PydanticValidationError, PolyaxonException, ValueError) as e:
                raise ValidationError(e)
        else:
            return create_run(
                project_id=project_id,
                user_id=user.id if user else None,
                name=name,
                description=description,
                tags=tags,
                meta_info=meta_info,
            )



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from rest_framework import fields, serializers

from haupt.db.abstracts.getter import get_lineage_model


class RunArtifactLightSerializer(serializers.ModelSerializer):
    name = fields.SerializerMethodField()
    kind = fields.SerializerMethodField()

    class Meta:
        model = get_lineage_model()
        fields = ("name", "kind", "is_input")

    def get_name(self, obj):
        return obj.artifact.name

    def get_kind(self, obj):
        return obj.artifact.kind


class RunArtifactBackwardCompatibleSerializer(RunArtifactLightSerializer):
    state = fields.SerializerMethodField()
    path = fields.SerializerMethodField()
    summary = fields.SerializerMethodField()

    class Meta(RunArtifactLightSerializer.Meta):
        fields = RunArtifactLightSerializer.Meta.fields + (
            "path",
            "summary",
            "state",
        )

    def get_state(self, obj):
        value = obj.artifact.state
        if value:
            return value.hex
        return value

    def get_path(self, obj):
        return obj.artifact.path

    def get_summary(self, obj):
        return obj.artifact.summary


class RunArtifactSerializer(RunArtifactBackwardCompatibleSerializer):
    run = fields.SerializerMethodField()
    meta_info = fields.SerializerMethodField()

    class Meta(RunArtifactBackwardCompatibleSerializer.Meta):
        fields = RunArtifactBackwardCompatibleSerializer.Meta.fields + (
            "run",
            "meta_info",
        )

    def get_run(self, obj):
        run = self.context.get("run")
        if not run:
            run = obj.run
        if run:
            return run.uuid.hex

    def get_meta_info(self, obj):
        run = self.context.get("run")
        if not run:
            run = obj.run
        if run:
            return {"run": {"name": run.name, "uuid": run.uuid.hex}}


class RunArtifactNameSerializer(serializers.ModelSerializer):
    name = fields.SerializerMethodField()

    class Meta:
        model = get_lineage_model()
        fields = ("name",)

    def get_name(self, obj):
        return obj.artifact.name



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from corsheaders.defaults import default_headers

from haupt.common.config_manager import ConfigManager
from polyaxon.services.headers import PolyaxonServiceHeaders


def set_cors(context, config: ConfigManager):
    # session settings
    context["CORS_ALLOW_CREDENTIALS"] = True
    allowed_list = config.get(
        "POLYAXON_CORS_ALLOWED_ORIGINS", "list", is_optional=True, default=[]
    )
    context["CORS_ALLOWED_ORIGINS"] = allowed_list
    context["CORS_ALLOW_ALL_ORIGINS"] = False if allowed_list else True

    context["CORS_ALLOW_HEADERS"] = (
        default_headers + PolyaxonServiceHeaders.get_headers()
    )

    ssl_enabled = config.get(
        "POLYAXON_SSL_ENABLED", "bool", is_optional=True, default=False
    )
    ssl_redirect_enabled = config.get(
        "POLYAXON_SSL_REDIRECT_ENABLED", "bool", is_optional=True, default=False
    )
    context["SSL_ENABLED"] = ssl_enabled
    context["PROTOCOL"] = "http"
    context["WS_PROTOCOL"] = "ws"
    if ssl_enabled:
        context["SESSION_COOKIE_SECURE"] = True
        context["CSRF_COOKIE_SECURE"] = True
        context["SECURE_PROXY_SSL_HEADER"] = ("HTTP_X_FORWARDED_PROTO", "https")
        context["PROTOCOL"] = "https"
        context["WS_PROTOCOL"] = "wss"
    if ssl_redirect_enabled:
        context["SECURE_SSL_REDIRECT"] = True



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.
from typing import List, Tuple
from urllib.parse import urlparse

from rest_framework import status

from django.http import HttpResponse

from polyaxon import settings
from polyaxon.api import K8S_V1_LOCATION
from polyaxon.k8s.async_manager import AsyncK8SManager


def _check_exec(uri_path: List[str], query_params: str):
    pod, container = uri_path
    query_params += "&container={}".format(container)
    path = "api/v1/namespaces/{namespace}/pods/{pod}/exec".format(
        namespace=settings.CLIENT_CONFIG.namespace,
        pod=pod,
    )
    return path, query_params


VALIDATION_PATHS = {
    "k8s_exec": _check_exec,
}


def k8s_check(uri: str) -> Tuple[str, str]:
    parsed_uri = urlparse(uri)
    uri_path = parsed_uri.path.split(K8S_V1_LOCATION)[-1].split("/")
    path_to_check = None
    for vpath in VALIDATION_PATHS:
        if vpath in uri_path:
            path_to_check = vpath
    if not path_to_check:
        raise ValueError("A valid k8s path param is required")
    start = uri_path.index(path_to_check) + 1
    return VALIDATION_PATHS[path_to_check](uri_path[start:], parsed_uri.query)


async def reverse_k8s(path) -> HttpResponse:
    if not path:
        return HttpResponse(
            content="A valid k8s path param is required",
            status=status.HTTP_400_BAD_REQUEST,
        )
    config = await AsyncK8SManager.load_config(
        in_cluster=settings.CLIENT_CONFIG.in_cluster
    )
    config_auth = AsyncK8SManager.get_config_auth(config)
    return HttpResponse(
        status=status.HTTP_200_OK,
        headers={
            "K8S_URI": "{}/{}".format(config.host, path),
            "K8S_TOKEN": config_auth,
        },
    )



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("db", "0003_run_pipeline"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="project",
            name="deleted",
        ),
        migrations.RemoveField(
            model_name="run",
            name="deleted",
        ),
        migrations.AddField(
            model_name="project",
            name="live_state",
            field=models.IntegerField(
                blank=True,
                choices=[(1, "live"), (0, "archived"), (-1, "deletion_progressing")],
                default=1,
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="run",
            name="live_state",
            field=models.IntegerField(
                blank=True,
                choices=[(1, "live"), (0, "archived"), (-1, "deletion_progressing")],
                default=1,
                null=True,
            ),
        ),
    ]



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.


class UniqueNameMixin:
    """
    A mixin to force setting a unique name.
    """

    @property
    def unique_name(self) -> str:
        raise NotImplementedError



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt.db.administration.utils import DiffModelAdmin


class ArtifactAdmin(DiffModelAdmin):
    list_display = (
        "name",
        "kind",
        "state",
    )
    list_display_links = (
        "name",
        "kind",
        "state",
    )
    readonly_fields = DiffModelAdmin.readonly_fields + ("name",)
    fields = ("name", "kind", "created_at", "updated_at")



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from django.conf import settings

from haupt.common.options import option_namespaces, option_subjects
from haupt.common.options.cache import LONG_CACHE_TTL
from haupt.common.options.option import (
    NAMESPACE_DB_OPTION_MARKER,
    Option,
    OptionScope,
    OptionStores,
)

STATS_DEFAULT_PREFIX = "{}{}{}".format(
    option_namespaces.STATS, NAMESPACE_DB_OPTION_MARKER, option_subjects.DEFAULT_PREFIX
)

OPTIONS = {STATS_DEFAULT_PREFIX}


class StatsDefaultPrefix(Option):
    key = STATS_DEFAULT_PREFIX
    scope = OptionScope.GLOBAL
    is_secret = False
    is_optional = True
    is_list = False
    typing = "str"
    store = OptionStores(settings.STORE_OPTION)
    default = None
    options = None
    cache_ttl = LONG_CACHE_TTL



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt.common.config_manager import ConfigManager


def set_encryption(context, config: ConfigManager):
    context["ENCRYPTION_KEY"] = config.get(
        "POLYAXON_ENCRYPTION_KEY", "str", is_optional=True
    )
    context["ENCRYPTION_SECRET"] = config.get(
        "POLYAXON_ENCRYPTION_SECRET", "str", is_optional=True
    )
    context["ENCRYPTION_BACKEND"] = config.get(
        "POLYAXON_ENCRYPTION_BACKEND", "str", is_optional=True
    )



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt.common.config_manager import ConfigManager


def set_middlewares(context, config: ConfigManager):
    context["MIDDLEWARE"] = (
        "django.middleware.security.SecurityMiddleware",
        "django.contrib.sessions.middleware.SessionMiddleware",
        "corsheaders.middleware.CorsMiddleware",
        "django.middleware.common.CommonMiddleware",
    )

    if not context["UI_IN_SANDBOX"]:
        context["MIDDLEWARE"] += ("django.middleware.csrf.CsrfViewMiddleware",)

    context["MIDDLEWARE"] += (
        "django.contrib.auth.middleware.AuthenticationMiddleware",
        "django.contrib.messages.middleware.MessageMiddleware",
        "django.middleware.clickjacking.XFrameOptionsMiddleware",
    )

    if context["UI_IN_SANDBOX"]:
        context["MIDDLEWARE"] += ("whitenoise.middleware.WhiteNoiseMiddleware",)


def set_base_middlewares(context, config: ConfigManager):
    context["MIDDLEWARE"] = ("django.middleware.common.CommonMiddleware",)
    if context["UI_IN_SANDBOX"]:
        context["MIDDLEWARE"] += ("whitenoise.middleware.WhiteNoiseMiddleware",)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

import datetime
import decimal
import uuid

from enum import Enum
from json import JSONEncoder, _default_decoder

from django.utils.html import mark_safe
from django.utils.timezone import is_aware


def better_default_encoder(o):
    if isinstance(o, uuid.UUID):
        return o.hex
    if isinstance(o, datetime.datetime):
        return o.strftime("%Y-%m-%dT%H:%M:%S.%fZ")
    if isinstance(o, datetime.date):
        return o.isoformat()
    if isinstance(o, datetime.time):
        if is_aware(o):
            raise ValueError("JSON can't represent timezone-aware times.")
        r = o.isoformat()
        if o.microsecond:
            r = r[:12]
        return r
    if isinstance(o, (set, frozenset)):
        return list(o)
    if isinstance(o, decimal.Decimal):
        return str(o)
    if isinstance(o, Enum):
        return o.value
    if callable(o):
        return "<function>"
    raise TypeError(repr(o) + " is not JSON serializable")


class JSONEncoderForHTML(JSONEncoder):
    def iterencode(self, o, _one_shot=False):
        chunks = super().iterencode(o, _one_shot)
        for chunk in chunks:
            chunk = chunk.replace("&", "\\u0026")
            chunk = chunk.replace("<", "\\u003c")
            chunk = chunk.replace(">", "\\u003e")
            chunk = chunk.replace("'", "\\u0027")
            yield chunk


_default_encoder = JSONEncoder(
    separators=(",", ":"),
    skipkeys=False,
    ensure_ascii=True,
    check_circular=True,
    allow_nan=True,
    indent=None,
    default=better_default_encoder,
)

_default_escaped_encoder = JSONEncoderForHTML(
    separators=(",", ":"),
    skipkeys=False,
    ensure_ascii=True,
    check_circular=True,
    allow_nan=True,
    indent=None,
    default=better_default_encoder,
)


def dump(value, fp, **kwargs):
    for chunk in _default_encoder.iterencode(value):
        fp.write(chunk)


def dumps(value, escape=False, **kwargs):
    # Prefer to use dumps_htmlsafe
    if escape:
        return _default_escaped_encoder.encode(value)
    return _default_encoder.encode(value)


def loads(value, **kwargs):
    return _default_decoder.decode(value)


def dumps_htmlsafe(value):
    return mark_safe(_default_escaped_encoder.encode(value))



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt.db.abstracts.projects import BaseProject


class Project(BaseProject):
    class Meta(BaseProject.Meta):
        app_label = "db"
        db_table = "db_project"



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.
from typing import Optional

from haupt.proxies.generators.base import write_to_conf_file
from haupt.proxies.schemas.server import get_server_config
from haupt.proxies.schemas.streams import get_base_config


def generate_streams_conf(path: Optional[str] = None, root: Optional[str] = None):
    write_to_conf_file(
        "polyaxon.main",
        get_server_config(root=root, use_upstream=True, use_redirect=False),
        path,
    )
    write_to_conf_file("polyaxon.base", get_base_config(), path)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

# isort: skip_file

# Default configs

from haupt.common.settings.logging import set_logging
from haupt.common.settings.admin import set_admin
from haupt.common.settings.secrets import set_secrets
from haupt.polyconf.config_manager import config

context = locals()
set_logging(context=context, config=config)
set_admin(context=context, config=config)
set_secrets(context=context, config=config)
if config.is_streams_service:
    from haupt.common.settings.services.streams import set_service
elif config.is_api_service:
    from haupt.common.settings.services.api import set_service
else:
    from haupt.common.settings.services.background import set_service

set_service(context=context, config=config)

from haupt.common.settings.defaults import *

if config.is_api_service or config.is_streams_service:
    from .rest import *



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

import re
import uuid

import django.contrib.auth.models
import django.contrib.auth.validators
import django.contrib.postgres.fields
import django.contrib.postgres.fields.jsonb
import django.core.serializers.json
import django.core.validators
import django.db.models.deletion
import django.utils.timezone

from django.conf import settings
from django.db import migrations, models

import haupt.common.validation.blacklist


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("auth", "0011_update_proxy_permissions"),
    ]

    operations = [
        migrations.CreateModel(
            name="User",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("password", models.CharField(max_length=128, verbose_name="password")),
                (
                    "last_login",
                    models.DateTimeField(
                        blank=True, null=True, verbose_name="last login"
                    ),
                ),
                (
                    "is_superuser",
                    models.BooleanField(
                        default=False,
                        help_text="Designates that this user has all permissions without explicitly assigning them.",
                        verbose_name="superuser status",
                    ),
                ),
                (
                    "username",
                    models.CharField(
                        error_messages={
                            "unique": "A user with that username already exists."
                        },
                        help_text="Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.",
                        max_length=150,
                        unique=True,
                        validators=[
                            django.contrib.auth.validators.UnicodeUsernameValidator()
                        ],
                        verbose_name="username",
                    ),
                ),
                (
                    "first_name",
                    models.CharField(
                        blank=True, max_length=30, verbose_name="first name"
                    ),
                ),
                (
                    "last_name",
                    models.CharField(
                        blank=True, max_length=150, verbose_name="last name"
                    ),
                ),
                (
                    "email",
                    models.EmailField(
                        blank=True, max_length=254, verbose_name="email address"
                    ),
                ),
                (
                    "is_staff",
                    models.BooleanField(
                        default=False,
                        help_text="Designates whether the user can log into this admin site.",
                        verbose_name="staff status",
                    ),
                ),
                (
                    "is_active",
                    models.BooleanField(
                        default=True,
                        help_text="Designates whether this user should be treated as active. Unselect this instead of deleting accounts.",
                        verbose_name="active",
                    ),
                ),
                (
                    "date_joined",
                    models.DateTimeField(
                        default=django.utils.timezone.now, verbose_name="date joined"
                    ),
                ),
            ],
            options={
                "verbose_name": "user",
                "verbose_name_plural": "users",
                "db_table": "db_user",
                "abstract": False,
                "swappable": "AUTH_USER_MODEL",
            },
            managers=[
                ("objects", django.contrib.auth.models.UserManager()),
            ],
        ),
        migrations.CreateModel(
            name="Artifact",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("state", models.UUIDField()),
                ("name", models.CharField(db_index=True, max_length=64)),
                (
                    "kind",
                    models.CharField(
                        choices=[
                            ("model", "model"),
                            ("audio", "audio"),
                            ("video", "video"),
                            ("histogram", "histogram"),
                            ("image", "image"),
                            ("tensor", "tensor"),
                            ("dataframe", "dataframe"),
                            ("chart", "chart"),
                            ("csv", "csv"),
                            ("tsv", "tsv"),
                            ("psv", "psv"),
                            ("ssv", "ssv"),
                            ("metric", "metric"),
                            ("env", "env"),
                            ("html", "html"),
                            ("text", "text"),
                            ("file", "file"),
                            ("dir", "dir"),
                            ("dockerfile", "dockerfile"),
                            ("docker_image", "docker_image"),
                            ("data", "data"),
                            ("coderef", "coderef"),
                            ("table", "table"),
                            ("tensorboard", "tensorboard"),
                            ("curve", "curve"),
                            ("confusion", "confusion"),
                            ("analysis", "analysis"),
                            ("iteration", "iteration"),
                            ("markdown", "markdown"),
                            ("system", "system"),
                            ("artifact", "artifact"),
                        ],
                        db_index=True,
                        max_length=12,
                    ),
                ),
                ("path", models.CharField(blank=True, max_length=256, null=True)),
                ("summary", django.contrib.postgres.fields.jsonb.JSONField()),
            ],
            options={
                "db_table": "db_artifact",
            },
        ),
        migrations.CreateModel(
            name="ArtifactLineage",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("is_input", models.NullBooleanField(default=False)),
            ],
            options={
                "db_table": "db_artifactlineage",
            },
        ),
        migrations.CreateModel(
            name="Project",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("deleted", models.BooleanField(default=False)),
                ("description", models.TextField(blank=True, null=True)),
                ("readme", models.TextField(blank=True, null=True)),
                (
                    "tags",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.CharField(max_length=64),
                        blank=True,
                        null=True,
                        size=None,
                    ),
                ),
                (
                    "uuid",
                    models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
                ),
                (
                    "name",
                    models.CharField(
                        max_length=150,
                        unique=True,
                        validators=[
                            django.core.validators.RegexValidator(
                                re.compile("^[-a-zA-Z0-9_]+\\Z"),
                                "Enter a valid “slug” consisting of letters, numbers, underscores or hyphens.",
                                "invalid",
                            ),
                            haupt.common.validation.blacklist.validate_blacklist_name,
                        ],
                    ),
                ),
            ],
            options={
                "db_table": "db_project",
            },
        ),
        migrations.CreateModel(
            name="Run",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("deleted", models.BooleanField(default=False)),
                ("description", models.TextField(blank=True, null=True)),
                (
                    "name",
                    models.CharField(
                        blank=True,
                        default=None,
                        max_length=128,
                        null=True,
                        validators=[
                            django.core.validators.RegexValidator(
                                re.compile("^[-a-zA-Z0-9_]+\\Z"),
                                "Enter a valid “slug” consisting of letters, numbers, underscores or hyphens.",
                                "invalid",
                            ),
                            haupt.common.validation.blacklist.validate_blacklist_name,
                        ],
                    ),
                ),
                ("readme", models.TextField(blank=True, null=True)),
                (
                    "tags",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.CharField(max_length=64),
                        blank=True,
                        null=True,
                        size=None,
                    ),
                ),
                (
                    "uuid",
                    models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
                ),
                (
                    "is_managed",
                    models.BooleanField(
                        default=True,
                        help_text="If this entity is managed by the platform.",
                    ),
                ),
                ("started_at", models.DateTimeField(blank=True, null=True)),
                ("finished_at", models.DateTimeField(blank=True, null=True)),
                ("run_time", models.IntegerField(blank=True, null=True)),
                (
                    "raw_content",
                    models.TextField(
                        blank=True,
                        help_text="The raw yaml content of the polyaxonfile/specification.",
                        null=True,
                    ),
                ),
                (
                    "content",
                    models.TextField(
                        blank=True,
                        help_text="The compiled yaml content of the polyaxonfile/specification.",
                        null=True,
                    ),
                ),
                (
                    "status",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("created", "created"),
                            ("resuming", "resuming"),
                            ("on_schedule", "on_schedule"),
                            ("compiled", "compiled"),
                            ("queued", "queued"),
                            ("scheduled", "scheduled"),
                            ("starting", "starting"),
                            ("running", "running"),
                            ("processing", "processing"),
                            ("stopping", "stopping"),
                            ("failed", "failed"),
                            ("stopped", "stopped"),
                            ("succeeded", "succeeded"),
                            ("skipped", "skipped"),
                            ("warning", "warning"),
                            ("unschedulable", "unschedulable"),
                            ("upstream_failed", "upstream_failed"),
                            ("retrying", "retrying"),
                            ("unknown", "unknown"),
                            ("done", "done"),
                        ],
                        db_index=True,
                        default="created",
                        max_length=64,
                        null=True,
                    ),
                ),
                (
                    "status_conditions",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        blank=True,
                        default=dict,
                        encoder=django.core.serializers.json.DjangoJSONEncoder,
                        null=True,
                    ),
                ),
                (
                    "kind",
                    models.CharField(
                        choices=[
                            ("job", "job"),
                            ("service", "service"),
                            ("dag", "dag"),
                            ("spark", "spark"),
                            ("dask", "dask"),
                            ("flink", "flink"),
                            ("ray", "ray"),
                            ("mpijob", "mpijob"),
                            ("tfjob", "tfjob"),
                            ("pytorchjob", "pytorchjob"),
                            ("paddlejob", "paddlejob"),
                            ("mxjob", "mxjob"),
                            ("xgbjob", "xgbjob"),
                            ("matrix", "matrix"),
                            ("schedule", "schedule"),
                            ("tuner", "tuner"),
                            ("watchdog", "watchdog"),
                            ("notifier", "notifier"),
                            ("cleaner", "cleaner"),
                            ("builder", "builder"),
                        ],
                        db_index=True,
                        max_length=12,
                    ),
                ),
                (
                    "meta_info",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        blank=True, default=dict, null=True
                    ),
                ),
                (
                    "params",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        blank=True, null=True
                    ),
                ),
                (
                    "inputs",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        blank=True, null=True
                    ),
                ),
                (
                    "outputs",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        blank=True, null=True
                    ),
                ),
                (
                    "cloning_kind",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("copy", "copy"),
                            ("restart", "restart"),
                            ("cache", "cache"),
                        ],
                        max_length=12,
                        null=True,
                    ),
                ),
                (
                    "artifacts",
                    models.ManyToManyField(
                        blank=True,
                        related_name="runs",
                        through="db.ArtifactLineage",
                        to="db.Artifact",
                    ),
                ),
                (
                    "original",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="clones",
                        to="db.Run",
                    ),
                ),
                (
                    "project",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="runs",
                        to="db.Project",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "db_table": "db_run",
            },
        ),
        migrations.AddIndex(
            model_name="project",
            index=models.Index(fields=["name"], name="db_project_name_4bfc0e_idx"),
        ),
        migrations.AddField(
            model_name="artifactlineage",
            name="artifact",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="runs_lineage",
                to="db.Artifact",
            ),
        ),
        migrations.AddField(
            model_name="artifactlineage",
            name="run",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="artifacts_lineage",
                to="db.Run",
            ),
        ),
        migrations.AlterUniqueTogether(
            name="artifact",
            unique_together={("name", "state")},
        ),
        migrations.AddField(
            model_name="user",
            name="groups",
            field=models.ManyToManyField(
                blank=True,
                help_text="The groups this user belongs to. A user will get all permissions granted to each of their groups.",
                related_name="user_set",
                related_query_name="user",
                to="auth.Group",
                verbose_name="groups",
            ),
        ),
        migrations.AddField(
            model_name="user",
            name="user_permissions",
            field=models.ManyToManyField(
                blank=True,
                help_text="Specific permissions for this user.",
                related_name="user_set",
                related_query_name="user",
                to="auth.Permission",
                verbose_name="user permissions",
            ),
        ),
        migrations.AddIndex(
            model_name="run",
            index=models.Index(fields=["name"], name="db_run_name_47fc7c_idx"),
        ),
        migrations.AlterUniqueTogether(
            name="artifactlineage",
            unique_together={("run", "artifact", "is_input")},
        ),
    ]



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from django.db import models


class StateModel(models.Model):
    state = models.UUIDField(db_index=True)

    class Meta:
        abstract = True


class OptionalStateModel(models.Model):
    state = models.UUIDField(null=True, blank=True, db_index=True)

    class Meta:
        abstract = True



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt import settings
from haupt.proxies.schemas.base import get_config

OPTIONS = """
send_timeout {timeout};
keepalive_timeout {timeout};
uwsgi_read_timeout {timeout};
uwsgi_send_timeout {timeout};
client_header_timeout {timeout};
proxy_read_timeout {timeout};
keepalive_requests 10000;
"""


def get_timeout_config():
    return get_config(
        options=OPTIONS, indent=0, timeout=settings.PROXIES_CONFIG.nginx_timeout
    )



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.
from typing import List, Optional
from typing_extensions import Literal

from clipped.utils.logging import DEFAULT_LOGS_ROOT
from pydantic import Extra, Field, validator

from polyaxon.api import STATIC_V1
from polyaxon.contexts import paths as ctx_paths
from polyaxon.env_vars.keys import (
    EV_KEYS_ARCHIVES_ROOT,
    EV_KEYS_DNS_BACKEND,
    EV_KEYS_DNS_CUSTOM_CLUSTER,
    EV_KEYS_DNS_PREFIX,
    EV_KEYS_DNS_USE_RESOLVER,
    EV_KEYS_K8S_NAMESPACE,
    EV_KEYS_LOG_LEVEL,
    EV_KEYS_LOGS_ROOT,
    EV_KEYS_NGINX_INDENT_CHAR,
    EV_KEYS_NGINX_INDENT_WIDTH,
    EV_KEYS_NGINX_TIMEOUT,
    EV_KEYS_PROXY_API_HOST,
    EV_KEYS_PROXY_API_PORT,
    EV_KEYS_PROXY_API_TARGET_PORT,
    EV_KEYS_PROXY_API_USE_RESOLVER,
    EV_KEYS_PROXY_AUTH_ENABLED,
    EV_KEYS_PROXY_AUTH_EXTERNAL,
    EV_KEYS_PROXY_AUTH_USE_RESOLVER,
    EV_KEYS_PROXY_FORWARD_PROXY_HOST,
    EV_KEYS_PROXY_FORWARD_PROXY_KIND,
    EV_KEYS_PROXY_FORWARD_PROXY_PORT,
    EV_KEYS_PROXY_GATEWAY_HOST,
    EV_KEYS_PROXY_GATEWAY_PORT,
    EV_KEYS_PROXY_GATEWAY_TARGET_PORT,
    EV_KEYS_PROXY_HAS_FORWARD_PROXY,
    EV_KEYS_PROXY_NAMESPACES,
    EV_KEYS_PROXY_SERVICES_PORT,
    EV_KEYS_PROXY_SSL_ENABLED,
    EV_KEYS_PROXY_SSL_PATH,
    EV_KEYS_PROXY_STREAMS_HOST,
    EV_KEYS_PROXY_STREAMS_PORT,
    EV_KEYS_PROXY_STREAMS_TARGET_PORT,
    EV_KEYS_STATIC_ROOT,
    EV_KEYS_STATIC_URL,
    EV_KEYS_UI_ADMIN_ENABLED,
)
from polyaxon.schemas.base import BaseSchemaModel


class ProxiesConfig(BaseSchemaModel):
    _IDENTIFIER = "proxies"
    _DEFAULT_TARGET_PORT = 8000
    _DEFAULT_PORT = 80

    namespace: Optional[str] = Field(alias=EV_KEYS_K8S_NAMESPACE)
    namespaces: Optional[List[str]] = Field(alias=EV_KEYS_PROXY_NAMESPACES)
    gateway_port: Optional[int] = Field(
        alias=EV_KEYS_PROXY_GATEWAY_PORT, default=_DEFAULT_PORT
    )
    gateway_target_port: Optional[int] = Field(
        alias=EV_KEYS_PROXY_GATEWAY_TARGET_PORT, default=_DEFAULT_TARGET_PORT
    )
    gateway_host: Optional[str] = Field(
        alias=EV_KEYS_PROXY_GATEWAY_HOST, default="polyaxon-polyaxon-gateway"
    )
    streams_port: Optional[int] = Field(
        alias=EV_KEYS_PROXY_STREAMS_PORT, default=_DEFAULT_PORT
    )
    streams_target_port: Optional[int] = Field(
        alias=EV_KEYS_PROXY_STREAMS_TARGET_PORT, default=_DEFAULT_TARGET_PORT
    )
    streams_host: Optional[str] = Field(
        alias=EV_KEYS_PROXY_STREAMS_HOST, default="polyaxon-polyaxon-streams"
    )
    api_port: Optional[int] = Field(alias=EV_KEYS_PROXY_API_PORT, default=_DEFAULT_PORT)
    api_target_port: Optional[int] = Field(
        alias=EV_KEYS_PROXY_API_TARGET_PORT, default=_DEFAULT_TARGET_PORT
    )
    api_host: Optional[str] = Field(
        alias=EV_KEYS_PROXY_API_HOST, default="polyaxon-polyaxon-api"
    )
    api_use_resolver: Optional[bool] = Field(
        alias=EV_KEYS_PROXY_API_USE_RESOLVER, default=False
    )
    services_port: Optional[int] = Field(
        alias=EV_KEYS_PROXY_SERVICES_PORT, default=_DEFAULT_PORT
    )
    auth_enabled: Optional[bool] = Field(
        alias=EV_KEYS_PROXY_AUTH_ENABLED, default=False
    )
    auth_external: Optional[str] = Field(alias=EV_KEYS_PROXY_AUTH_EXTERNAL)
    auth_use_resolver: Optional[bool] = Field(
        alias=EV_KEYS_PROXY_AUTH_USE_RESOLVER, default=False
    )
    ssl_enabled: Optional[bool] = Field(alias=EV_KEYS_PROXY_SSL_ENABLED, default=False)
    ssl_path: Optional[str] = Field(
        alias=EV_KEYS_PROXY_SSL_PATH, default="/etc/ssl/polyaxon"
    )
    dns_use_resolver: Optional[bool] = Field(
        alias=EV_KEYS_DNS_USE_RESOLVER, default=False
    )
    dns_custom_cluster: Optional[str] = Field(
        alias=EV_KEYS_DNS_CUSTOM_CLUSTER, default="cluster.local"
    )
    dns_backend: Optional[str] = Field(alias=EV_KEYS_DNS_BACKEND, default="kube-dns")
    dns_prefix: Optional[str] = Field(alias=EV_KEYS_DNS_PREFIX)
    logs_root: Optional[str] = Field(alias=EV_KEYS_LOGS_ROOT, default=DEFAULT_LOGS_ROOT)
    log_level: Optional[str] = Field(alias=EV_KEYS_LOG_LEVEL)
    nginx_timeout: Optional[int] = Field(alias=EV_KEYS_NGINX_TIMEOUT, default=650)
    nginx_indent_char: Optional[str] = Field(
        alias=EV_KEYS_NGINX_INDENT_CHAR, default=" "
    )
    nginx_indent_width: Optional[int] = Field(
        alias=EV_KEYS_NGINX_INDENT_WIDTH, default=4
    )
    archives_root: Optional[str] = Field(
        alias=EV_KEYS_ARCHIVES_ROOT, default=ctx_paths.CONTEXT_ARCHIVES_ROOT
    )
    static_root: Optional[str] = Field(
        alias=EV_KEYS_STATIC_ROOT, default="/{}".format(STATIC_V1)
    )
    static_url: Optional[str] = Field(alias=EV_KEYS_STATIC_URL)
    ui_admin_enabled: Optional[bool] = Field(alias=EV_KEYS_UI_ADMIN_ENABLED)
    has_forward_proxy: Optional[bool] = Field(alias=EV_KEYS_PROXY_HAS_FORWARD_PROXY)
    forward_proxy_port: Optional[int] = Field(alias=EV_KEYS_PROXY_FORWARD_PROXY_PORT)
    forward_proxy_host: Optional[str] = Field(alias=EV_KEYS_PROXY_FORWARD_PROXY_HOST)
    forward_proxy_kind: Optional[Literal["transparent", "connect"]] = Field(
        alias=EV_KEYS_PROXY_FORWARD_PROXY_KIND
    )

    class Config:
        extra = Extra.ignore

    @validator("log_level", always=True)
    def validate_log_level(cls, v):
        return (v or "warn").lower()



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.
from rest_framework.exceptions import ValidationError
from rest_framework.response import Response

from django.http import Http404

from haupt.apis.endpoints.run import RunEndpoint
from haupt.apis.methods import runs as methods
from haupt.apis.serializers.runs import (
    RunDetailSerializer,
    RunSerializer,
    RunStatusSerializer,
)
from haupt.common import conf
from haupt.common.apis.regex import RUN_UUID_KEY
from haupt.common.endpoints.base import (
    CreateEndpoint,
    DestroyEndpoint,
    RetrieveEndpoint,
    UpdateEndpoint,
)
from haupt.common.events.registry.run import (
    RUN_APPROVED_ACTOR,
    RUN_COPIED_ACTOR,
    RUN_DELETED_ACTOR,
    RUN_RESTARTED_ACTOR,
    RUN_RESUMED_ACTOR,
    RUN_STOPPED_ACTOR,
)
from haupt.common.options.registry.k8s import K8S_NAMESPACE
from haupt.db.queries import runs as runs_queries
from haupt.db.queries.runs import STATUS_UPDATE_COLUMNS_DEFER
from haupt.orchestration import operations
from polyaxon.lifecycle import LifeCycle


class RunDetailView(RunEndpoint, RetrieveEndpoint, DestroyEndpoint, UpdateEndpoint):
    queryset = runs_queries.run_model.all.select_related("original", "project")
    serializer_class = RunDetailSerializer
    AUDITOR_EVENT_TYPES = {
        "DELETE": RUN_DELETED_ACTOR,
    }
    AUDIT_PROJECT_RESOURCES = True
    PROJECT_RESOURCE_KEY = RUN_UUID_KEY
    AUDIT_INSTANCE = True

    def perform_destroy(self, instance):
        # Deletion managed by the handler
        pass


class RunCloneView(RunEndpoint, CreateEndpoint):
    serializer_class = RunSerializer
    AUDIT_PROJECT_RESOURCES = True
    PROJECT_RESOURCE_KEY = RUN_UUID_KEY
    AUDIT_INSTANCE = True

    def clone(self, obj, content, **kwargs):
        pass

    def pre_validate(self, obj):
        return obj

    def post(self, request, *args, **kwargs):
        return methods.clone_run(view=self, request=request, *args, **kwargs)


class RunRestartView(RunCloneView):
    AUDITOR_EVENT_TYPES = {"POST": RUN_RESTARTED_ACTOR}

    def clone(self, obj, content, **kwargs):
        return operations.restart_run(
            run=obj,
            user_id=self.request.user.id,
            content=content,
            name=kwargs.get("name"),
            description=kwargs.get("description"),
            tags=kwargs.get("tags"),
        )


class RunResumeView(RunCloneView):
    AUDITOR_EVENT_TYPES = {"POST": RUN_RESUMED_ACTOR}

    def clone(self, obj, content, **kwargs):
        return operations.resume_run(
            run=obj,
            user_id=self.request.user.id,
            content=content,
            message="Run was resumed by user.",
        )

    def pre_validate(self, obj):
        if not LifeCycle.is_done(obj.status):
            raise ValidationError(
                "Cannot resume this run, the run must reach a final state first, "
                "current status error: {}".format(obj.status)
            )
        return super().pre_validate(obj)


class RunCopyView(RunCloneView):
    AUDITOR_EVENT_TYPES = {"POST": RUN_COPIED_ACTOR}

    def clone(self, obj, content, **kwargs):
        return operations.copy_run(
            run=obj,
            user_id=self.request.user.id,
            content=content,
            name=kwargs.get("name"),
            description=kwargs.get("description"),
            tags=kwargs.get("tags"),
            meta_info=kwargs.get("meta_info"),
        )


class RunStatusListView(RunEndpoint, RetrieveEndpoint, CreateEndpoint):
    queryset = runs_queries.run_model.restorable.defer(
        *STATUS_UPDATE_COLUMNS_DEFER
    ).select_related(
        "project",
    )
    serializer_class = RunStatusSerializer

    def perform_create(self, serializer):
        try:
            methods.create_status(view=self, serializer=serializer)
        except runs_queries.run_model.DoesNotExit:
            raise Http404


class RunStopView(RunEndpoint, CreateEndpoint):
    AUDITOR_EVENT_TYPES = {"POST": RUN_STOPPED_ACTOR}
    AUDIT_PROJECT_RESOURCES = True
    PROJECT_RESOURCE_KEY = RUN_UUID_KEY
    AUDIT_INSTANCE = True

    def post(self, request, *args, **kwargs):
        return methods.stop_run(view=self, request=request, *args, **kwargs)


class RunApproveView(RunEndpoint, CreateEndpoint):
    AUDITOR_EVENT_TYPES = {"POST": RUN_APPROVED_ACTOR}
    AUDIT_PROJECT_RESOURCES = True
    PROJECT_RESOURCE_KEY = RUN_UUID_KEY
    AUDIT_INSTANCE = True

    def post(self, request, *args, **kwargs):
        return methods.approve_run(view=self, request=request, *args, **kwargs)


class RunNamespaceView(RunEndpoint, RetrieveEndpoint):
    def retrieve(self, request, *args, **kwargs):
        namespace = {"namespace": conf.get(K8S_NAMESPACE)}
        return Response(namespace)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from rest_framework.generics import get_object_or_404

from django.http import HttpRequest

from haupt.common.apis.regex import (
    NAME_KEY,
    OWNER_NAME_KEY,
    PROJECT_NAME_KEY,
    PROJECT_OWNER_NAME_KEY,
    UUID_KEY,
)
from haupt.common.endpoints.base import BaseEndpoint
from haupt.db.queries import projects as projects_queries


class ProjectEndpoint(BaseEndpoint):
    queryset = projects_queries.project_model.objects
    lookup_field = NAME_KEY
    lookup_url_kwarg = PROJECT_NAME_KEY
    CONTEXT_KEYS = (OWNER_NAME_KEY, PROJECT_NAME_KEY)
    CONTEXT_OBJECTS = ("project",)

    PROJECT_NAME_KEY = "name"
    PROJECT_OWNER_NAME_KEY = OWNER_NAME_KEY

    def initialize_object_context(self, request: HttpRequest, *args, **kwargs):
        #  pylint:disable=attribute-defined-outside-init
        self.project = self.get_object()

    def set_owner(self):
        self._owner_id = self.project.owner.id


class ProjectResourceListEndpoint(ProjectEndpoint):
    AUDITOR_EVENT_TYPES = None

    def get_object(self):
        if self._object:
            return self._object
        self._object = get_object_or_404(
            projects_queries.project_model,
            name=self.project_name,
        )
        return self._object

    def enrich_queryset(self, queryset):
        return queryset.filter(project=self.project)


class ProjectResourceEndpoint(ProjectEndpoint):
    AUDITOR_EVENT_TYPES = None
    lookup_field = UUID_KEY

    PROJECT_NAME_KEY = PROJECT_NAME_KEY
    PROJECT_OWNER_NAME_KEY = PROJECT_OWNER_NAME_KEY



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt import settings
from haupt.proxies.schemas.base import get_config

HTTP_OPTIONS = """
listen {port};
"""

SSL_OPTIONS = """
listen 443 ssl;
ssl on;
"""


def get_listen_config(is_proxy: bool, port: int = 8000) -> str:
    options = (
        SSL_OPTIONS
        if is_proxy and settings.PROXIES_CONFIG.ssl_enabled
        else HTTP_OPTIONS
    )
    return get_config(options=options, indent=0, port=port)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from typing import Tuple

from clipped.manager_interface import ManagerInterface

from haupt.common.options.option import Option


class OptionManager(ManagerInterface):
    def _get_state_data(  # pylint:disable=arguments-differ
        self, option: Option
    ) -> Tuple[str, Option]:
        return option.key, option

    def subscribe(self, option: Option):  # pylint:disable=arguments-differ
        """
        >>> subscribe(SomeOption)
        """
        super().subscribe(obj=option)

    def get(self, key: str) -> Option:  # pylint:disable=arguments-differ
        return super().get(key=key)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

import factory

from faker import Factory as FakerFactory

from django.conf import settings

fake = FakerFactory.create()


class UserFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = settings.AUTH_USER_MODEL

    username = factory.LazyAttribute(lambda x: fake.user_name())
    first_name = factory.LazyAttribute(lambda x: fake.first_name())
    last_name = factory.LazyAttribute(lambda x: fake.last_name())
    email = factory.LazyAttribute(lambda x: fake.email())
    password = factory.PostGenerationMethodCall("set_password", "defaultpassword")
    is_staff = False
    is_superuser = False



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from typing import Dict, Optional

from rest_framework import serializers

from haupt.db.abstracts.runs import BaseRun


class CloningMixin(serializers.Serializer):
    def get_original(self, obj: BaseRun) -> Optional[Dict]:
        if not obj.original_id:
            return None

        return {
            "uuid": obj.original.uuid.hex,
            "name": obj.original.name,
            "kind": obj.cloning_kind,
        }



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt.common.apis.regex import OWNER_NAME_PATTERN, PROJECT_NAME_PATTERN

# Projects
URLS_PROJECTS_CREATE = r"^{}/projects/create/?$".format(OWNER_NAME_PATTERN)
URLS_PROJECTS_LIST = r"^{}/projects/list/?$".format(OWNER_NAME_PATTERN)
URLS_PROJECTS_NAMES = r"^{}/projects/names/?$".format(OWNER_NAME_PATTERN)
URLS_PROJECTS_DETAILS = r"^{}/{}/?$".format(OWNER_NAME_PATTERN, PROJECT_NAME_PATTERN)

# Resources
URLS_PROJECTS_RUNS_TAG = r"^{}/{}/runs/tag/?$".format(
    OWNER_NAME_PATTERN, PROJECT_NAME_PATTERN
)
URLS_PROJECTS_RUNS_STOP = r"^{}/{}/runs/stop/?$".format(
    OWNER_NAME_PATTERN, PROJECT_NAME_PATTERN
)
URLS_PROJECTS_RUNS_APPROVE = r"^{}/{}/runs/approve/?$".format(
    OWNER_NAME_PATTERN, PROJECT_NAME_PATTERN
)
URLS_PROJECTS_RUNS_DELETE = r"^{}/{}/runs/delete/?$".format(
    OWNER_NAME_PATTERN, PROJECT_NAME_PATTERN
)
URLS_PROJECTS_RUNS_SYNC = r"^{}/{}/runs/sync/?$".format(
    OWNER_NAME_PATTERN, PROJECT_NAME_PATTERN
)
URLS_PROJECTS_RUNS_ARTIFACTS_LINEAGE_V0 = r"^{}/{}/runs/artifacts_lineage/?$".format(
    OWNER_NAME_PATTERN, PROJECT_NAME_PATTERN
)
URLS_PROJECTS_RUNS_ARTIFACTS_LINEAGE = r"^{}/{}/runs/lineage/artifacts/?$".format(
    OWNER_NAME_PATTERN, PROJECT_NAME_PATTERN
)
URLS_PROJECTS_RUNS_LIST = r"^{}/{}/runs/?$".format(
    OWNER_NAME_PATTERN, PROJECT_NAME_PATTERN
)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from django.core.management import BaseCommand


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument(
            "--app",
            dest="app",
            default="coredb",
        )

    @staticmethod
    def update_migrations(cursor, app):
        query = "UPDATE django_migrations SET app = 'db' where app='{}'".format(app)
        cursor.execute(query)

    @staticmethod
    def update_table(cursor):
        check_query = (
            "select * from information_schema.tables where table_name='{}'".format(
                "auth_user"
            )
        )
        cursor.execute(check_query)
        if bool(cursor.fetchall()):
            alter_query = "ALTER TABLE auth_user RENAME TO db_user"
            cursor.execute(alter_query)

    def handle(self, *args, **options):
        from django.conf import settings
        from django.db import connection

        if settings.DB_ENGINE_NAME == "sqlite":
            return

        with connection.cursor() as cursor:
            self.update_migrations(cursor, options["app"])



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.
from typing import Optional

from haupt.proxies.generators.base import write_to_conf_file
from haupt.proxies.schemas.gateway import get_base_config, get_redirect_config
from haupt.proxies.schemas.server import get_server_config


def generate_gateway_conf(
    path: Optional[str] = None, root: Optional[str] = None, is_platform: bool = True
):
    write_to_conf_file(
        "polyaxon.main",
        get_server_config(root=root, use_upstream=is_platform, use_redirect=True),
        path,
    )
    write_to_conf_file("polyaxon.base", get_base_config(is_platform=is_platform), path)
    write_to_conf_file("polyaxon.redirect", get_redirect_config(), path)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from rest_framework.urlpatterns import format_suffix_patterns

from django.urls import re_path

from haupt.apis.project_resources import views
from haupt.common.apis.urls import projects

projects_urlpatterns = [
    re_path(
        projects.URLS_PROJECTS_RUNS_TAG,
        views.ProjectRunsTagView.as_view(),
    ),
    re_path(
        projects.URLS_PROJECTS_RUNS_ARTIFACTS_LINEAGE_V0,
        views.ProjectRunsArtifactsView.as_view(),
    ),
    re_path(
        projects.URLS_PROJECTS_RUNS_ARTIFACTS_LINEAGE,
        views.ProjectRunsArtifactsView.as_view(),
    ),
    re_path(
        projects.URLS_PROJECTS_RUNS_STOP,
        views.ProjectRunsStopView.as_view(),
    ),
    re_path(
        projects.URLS_PROJECTS_RUNS_APPROVE,
        views.ProjectRunsApproveView.as_view(),
    ),
    re_path(
        projects.URLS_PROJECTS_RUNS_SYNC,
        views.ProjectRunsSyncView.as_view(),
    ),
    re_path(
        projects.URLS_PROJECTS_RUNS_DELETE,
        views.ProjectRunsDeleteView.as_view(),
    ),
    re_path(
        projects.URLS_PROJECTS_RUNS_LIST,
        views.ProjectRunsListView.as_view(),
    ),
]

# Order is important, because the patterns could swallow other urls
urlpatterns = format_suffix_patterns(projects_urlpatterns)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt.common import auditor
from haupt.common.events.registry import run

auditor.subscribe(run.RunCreatedEvent)
auditor.subscribe(run.RunSyncedActorEvent)
auditor.subscribe(run.RunResumedEvent)
auditor.subscribe(run.RunStoppedEvent)
auditor.subscribe(run.RunSkippedEvent)
auditor.subscribe(run.RunNewStatusEvent)
auditor.subscribe(run.RunNewArtifactsEvent)
auditor.subscribe(run.RunSucceededEvent)
auditor.subscribe(run.RunFailedEvent)
auditor.subscribe(run.RunDoneEvent)
auditor.subscribe(run.RunCreatedActorEvent)
auditor.subscribe(run.RunUpdatedActorEvent)
auditor.subscribe(run.RunDeletedActorEvent)
auditor.subscribe(run.RunViewedActorEvent)
auditor.subscribe(run.RunStoppedActorEvent)
auditor.subscribe(run.RunTransferredActorEvent)
auditor.subscribe(run.RunApprovedActorEvent)
auditor.subscribe(run.RunInvalidatedActorEvent)
auditor.subscribe(run.RunResumedActorEvent)
auditor.subscribe(run.RunRestartedActorEvent)
auditor.subscribe(run.RunSyncedActorEvent)
auditor.subscribe(run.RunCopiedActorEvent)
auditor.subscribe(run.RunSkippedActorEvent)
auditor.subscribe(run.RunStatsActorEvent)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

import inspect
import itertools

from clipped.utils.imports import import_string

try:
    from django.utils.functional import LazyObject, empty  # pylint:disable=import-error
except ImportError:
    raise ImportError("This module depends on django.")


class InvalidService(Exception):
    pass


class Service:
    __all__ = ()
    is_setup = False

    def validate(self):
        """Validate the settings for this backend (i.e. such as proper connection info).

        Raise ``InvalidService`` if there is a configuration error.
        """

    def setup(self):
        """Initialize this service."""
        self.is_setup = True

    def validate_and_setup(self):
        self.validate()
        self.setup()


class LazyServiceWrapper(LazyObject):
    """Lazyily instantiates a Polyaxon standard service class.

    >>> LazyServiceWrapper(BaseClass, 'path.to.import.Backend', {})

    Provides an ``expose`` method for dumping public APIs to a context, such as module locals:

    >>> service = LazyServiceWrapper(...)
    >>> service.expose(locals())
    """

    def __init__(self, backend_base, backend_path, options):
        super().__init__()
        self.__dict__.update(
            {
                "_backend_base": backend_base,
                "_backend_path": backend_path,
                "_options": options,
            }
        )

    def __getattr__(self, name):
        if self._wrapped is empty:  # pylint:disable=attribute-defined-outside-init
            self._setup()
        return getattr(self._wrapped, name)

    def _setup(self):
        backend = import_string(self._backend_path)
        assert issubclass(backend, Service)
        instance = backend(**self._options)
        self._wrapped = instance  # pylint:disable=attribute-defined-outside-init

    def expose(self, context):
        base = self._backend_base
        for key in itertools.chain(
            base.__all__, ("validate", "setup", "validate_and_setup")
        ):
            if inspect.isfunction(getattr(base, key)):
                # pylint:disable=unnecessary-lambda
                # pylint:disable=undefined-variable
                context[key] = (lambda f: lambda *a, **k: getattr(self, f)(*a, **k))(
                    key
                )
            else:
                context[key] = getattr(base, key)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from clipped.utils.json import orjson_dumps
from rest_framework import status

from django.core.handlers.asgi import ASGIRequest
from django.http import HttpResponse
from django.urls import re_path


async def health(request: ASGIRequest) -> HttpResponse:
    return HttpResponse(status=status.HTTP_200_OK)


class UJSONResponse(HttpResponse):
    def __init__(
        self,
        data,
        safe=True,
        json_dumps_params=None,
        **kwargs,
    ):
        if safe and not isinstance(data, dict):
            raise TypeError(
                "In order to allow non-dict objects to be serialized set the "
                "safe parameter to False."
            )
        if json_dumps_params is None:
            json_dumps_params = {}
        kwargs.setdefault("content_type", "application/json")
        data = orjson_dumps(data, **json_dumps_params)
        super().__init__(content=data, **kwargs)


class ConfigResponse(HttpResponse):
    def __init__(
        self,
        data,
        **kwargs,
    ):
        kwargs.setdefault("content_type", "application/json")
        super().__init__(content=data, **kwargs)


base_health_route = re_path(r"^healthz/?$", health, name="health_check")



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt.proxies.schemas.base import get_config

OPTIONS = """
gzip                        on;
gzip_disable                "msie6";
gzip_types                  *;
gzip_proxied                any;
"""


def get_gzip_config():
    return get_config(options=OPTIONS, indent=0)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from django.http import HttpResponse
from django.views.generic import View


class StatusView(View):
    status_code = 200

    def get(self, request, *args, **kwargs):
        return HttpResponse(status=self.status_code)


class Handler404View(StatusView):
    status_code = 404


class Handler50xView(StatusView):
    status_code = 500


class Handler403View(StatusView):
    status_code = 403



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.


class CeleryCoreQueues:
    """Celery Core Queues.

    N.B. make sure that the queue name is not < 128.
    """

    SCHEDULER_HEALTH = "queues.scheduler.health"
    SCHEDULER_RUNS = "queues.scheduler.runs"
    SCHEDULER_COMPILER = "queues.scheduler.compiler"
    SCHEDULER_ARTIFACTS = "queues.scheduler.artifacts"
    SCHEDULER_CLEAN = "queues.scheduler.clean"

    ALL_QUEUES = (
        SCHEDULER_HEALTH,
        SCHEDULER_RUNS,
        SCHEDULER_COMPILER,
        SCHEDULER_ARTIFACTS,
        SCHEDULER_CLEAN,
    )



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from typing import Any, Optional, Tuple

from haupt.common.service_interface import Service
from polyaxon.exceptions import PQLException
from polyaxon.pql.manager import PQLManager
from polyaxon.pql.parser import parse_field


class QueryService(Service):
    __all__ = ("filter_queryset", "parse_field")

    @classmethod
    def filter_queryset(
        cls, manager: PQLManager, query_spec: str, queryset: Any
    ) -> Any:
        try:
            return manager.apply(query_spec=query_spec, queryset=queryset)
        except Exception as e:
            raise PQLException("Error applying or resolving queryset, %s" % e)

    @classmethod
    def parse_field(cls, field: str) -> Tuple[str, Optional[str]]:
        return parse_field(field=field)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from functools import wraps

from django.utils.text import compress_string

from haupt.polyconf.config_manager import config


class GzipDecorator:
    """Gzip the response and set the respective header."""

    def __call__(self, func):
        @wraps(func)
        def inner(self, request, *args, **kwargs):
            response = func(self, request, *args, **kwargs)

            if (
                config
                and config.is_debug_mode
                and config.is_monolith_service
                and not config.is_test_env
            ):
                return response

            # Before we can access response.content, the response needs to be rendered.
            response = self.finalize_response(request, response, *args, **kwargs)
            response.render()  # should be rendered, before picklining while storing to cache

            compressed_content = compress_string(response.content)

            # Ensure that the compressed content is actually smaller than the original.
            if len(compressed_content) >= len(response.content):
                return response

            # Replace content with gzipped variant, update respective headers.
            response.content = compressed_content
            response["Content-Length"] = str(len(response.content))
            response["Content-Encoding"] = "gzip"

            return response

        return inner


gzip = GzipDecorator



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("db", "0009_project_unique_name"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="run",
            name="is_approved",
        ),
        migrations.AddField(
            model_name="run",
            name="pending",
            field=models.CharField(
                blank=True,
                choices=[
                    ("approval", "approval"),
                    ("upload", "upload"),
                    ("cache", "cache"),
                    ("build", "build"),
                ],
                db_index=True,
                help_text="If this entity requires approval before it should run.",
                max_length=8,
                null=True,
            ),
        ),
    ]



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

import datetime
import uuid

from collections.abc import Mapping
from urllib.parse import urlparse

from clipped.utils.json import orjson_dumps

from django.test import Client
from django.test.client import FakePayload

CONTENT_TYPE_APPLICATION_JSON = "application/json"


class BaseClient(Client):
    """Base client class."""

    def do_request(
        self,
        method,
        path,
        data=None,
        content_type=CONTENT_TYPE_APPLICATION_JSON,
        **extra
    ):
        if data is None:
            data = {}

        def validate_data(dvalues):
            if not isinstance(dvalues, Mapping):
                return
            for key, value in dvalues.items():
                # Fix UUIDs for convenience
                if isinstance(value, uuid.UUID):
                    dvalues[key] = value.hex

                # Fix datetimes
                if isinstance(value, datetime.datetime):
                    dvalues[key] = value.strftime("%Y-%m-%d %H:%M")

        if isinstance(data, list):
            for d in data:
                validate_data(d)
        else:
            validate_data(data)

        if content_type == CONTENT_TYPE_APPLICATION_JSON:
            data = orjson_dumps(data)

        request = self.encode_data(method, path, data, content_type, **extra)
        return self.request(**request)

    def put(self, path, data=None, content_type=CONTENT_TYPE_APPLICATION_JSON, **extra):
        """Construct a PUT request."""
        return self.do_request("PUT", path, data, content_type, **extra)

    def patch(
        self, path, data=None, content_type=CONTENT_TYPE_APPLICATION_JSON, **extra
    ):
        """Construct a PATCH request."""
        return self.do_request("PATCH", path, data, content_type, **extra)

    def post(
        self, path, data=None, content_type=CONTENT_TYPE_APPLICATION_JSON, **extra
    ):
        """Construct a PATCH request."""
        return self.do_request("POST", path, data, content_type, **extra)

    def delete(
        self, path, data=None, content_type=CONTENT_TYPE_APPLICATION_JSON, **extra
    ):
        """Construct a DELETE request."""
        return self.do_request("DELETE", path, data, content_type, **extra)

    def encode_data(self, http_method, path, data, content_type, **extra):
        patch_data = self._encode_data(data, content_type)

        parsed = urlparse(path)
        request = {
            "CONTENT_LENGTH": len(patch_data),
            "CONTENT_TYPE": content_type,
            "PATH_INFO": self._get_path(parsed),
            "QUERY_STRING": parsed[4],
            "REQUEST_METHOD": http_method,
            "wsgi.input": FakePayload(patch_data),
        }
        request.update(extra)

        return request



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

# isort: skip_file

from django.apps import AppConfig


class StreamsConfig(AppConfig):
    name = "haupt.streams"
    verbose_name = "Streams"

    def ready(self):
        from haupt.common import conf

        conf.validate_and_setup()

        import haupt.common.options.conf_subscriptions  # noqa



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

import os

from haupt.schemas.proxies_config import ProxiesConfig
from polyaxon.config.manager import ConfigManager
from polyaxon.managers.base import BaseConfigManager, ManagerVisibility


class ProxiesManager(BaseConfigManager):
    """Manages proxies configuration file."""

    VISIBILITY = ManagerVisibility.GLOBAL
    CONFIG_FILE_NAME = ".proxies"
    CONFIG = ProxiesConfig

    @classmethod
    def get_config_from_env(cls, **kwargs) -> ProxiesConfig:
        config_paths = [os.environ, {"dummy": "dummy"}]

        proxy_config = ConfigManager.read_configs(config_paths)
        return ProxiesConfig.from_dict(proxy_config.data)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

FEATURES = "FEATURES"
SCHEDULER = "SCHEDULER"
STATS = "STATS"
TTL = "TTL"
SIDECARS = "SIDECARS"
INIT = "INIT"
CLEANING_INTERVALS = "CLEANING_INTERVALS"
COMPONENT = "COMPONENT"
TIMEOUT = "TIMEOUT"



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt.common.config_manager import ConfigManager


def set_admin(context, config: ConfigManager):
    admin_name = config.get("POLYAXON_ADMIN_NAME", "str", is_optional=True)
    admin_mail = config.get("POLYAXON_ADMIN_MAIL", "str", is_optional=True)

    if admin_mail and admin_mail:
        admins = ((admin_name, admin_mail),)
        context["ADMINS"] = admins
        context["MANAGERS"] = admins



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from polyaxon.fs.fs import get_default_fs
from polyaxon.fs.types import FSSystem


class AppFS:
    FS = None

    @classmethod
    async def set_fs(cls) -> FSSystem:
        fs = await get_default_fs()
        cls.FS = fs
        return cls.FS

    @classmethod
    async def close_fs(cls):
        if cls.FS and hasattr(cls.FS, "close_session"):
            cls.FS.close_session(cls.FS.loop, cls.FS.session)

    @classmethod
    async def get_fs(cls) -> FSSystem:
        if not cls.FS:
            return await cls.set_fs()
        return cls.FS



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.
from rest_framework.generics import CreateAPIView

from haupt.apis.endpoints.project import ProjectEndpoint
from haupt.apis.serializers.projects import (
    ProjectCreateSerializer,
    ProjectDetailSerializer,
    ProjectNameSerializer,
    ProjectSerializer,
)
from haupt.common.apis.filters import OrderingFilter, QueryFilter
from haupt.common.apis.paginator import LargeLimitOffsetPagination
from haupt.common.endpoints.base import (
    BaseEndpoint,
    DestroyEndpoint,
    ListEndpoint,
    RetrieveEndpoint,
    UpdateEndpoint,
)
from haupt.db.queries import projects as projects_queries
from haupt.db.query_managers.project import ProjectQueryManager


class ProjectCreateView(BaseEndpoint, CreateAPIView):
    serializer_class = ProjectCreateSerializer


class ProjectListView(BaseEndpoint, ListEndpoint):
    queryset = projects_queries.project_model.all.only(
        "uuid",
        "name",
        "description",
        "created_at",
        "updated_at",
        "tags",
    ).order_by("-updated_at")
    serializer_class = ProjectSerializer

    filter_backends = (QueryFilter, OrderingFilter)
    query_manager = ProjectQueryManager
    check_alive = ProjectQueryManager.CHECK_ALIVE
    ordering = ProjectQueryManager.FIELDS_DEFAULT_ORDERING
    ordering_fields = ProjectQueryManager.FIELDS_ORDERING
    ordering_proxy_fields = ProjectQueryManager.FIELDS_ORDERING_PROXY


class ProjectNameListView(BaseEndpoint, ListEndpoint):
    queryset = projects_queries.project_model.objects.only("name").order_by(
        "-updated_at"
    )
    serializer_class = ProjectNameSerializer
    pagination_class = LargeLimitOffsetPagination


class ProjectDetailView(
    ProjectEndpoint, RetrieveEndpoint, UpdateEndpoint, DestroyEndpoint
):
    queryset = projects_queries.project_model.all
    serializer_class = ProjectDetailSerializer



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt import settings
from haupt.proxies.schemas.base import get_config

SSL_REDIRECT_OPTIONS = r"""
server {{
    listen 80;
    return 301 https://$host$request_uri;
}}
"""


def get_redirect_config():
    return get_config(
        options=SSL_REDIRECT_OPTIONS if settings.PROXIES_CONFIG.ssl_enabled else "",
        indent=0,
        ssl_path=settings.PROXIES_CONFIG.ssl_path,
    )



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from django.urls import include, re_path

from haupt.common.apis.index import get_urlpatterns, handler403, handler404, handler500
from haupt.common.apis.regex import OWNER_NAME_PATTERN, PROJECT_NAME_PATTERN
from haupt.streams.endpoints.artifacts import artifacts_routes
from haupt.streams.endpoints.auth_request import auth_request_routes
from haupt.streams.endpoints.events import events_routes
from haupt.streams.endpoints.k8s import k8s_routes
from haupt.streams.endpoints.logs import internal_logs_routes, logs_routes
from haupt.streams.endpoints.notifications import notifications_routes
from polyaxon.api import API_V1, AUTH_REQUEST_V1, INTERNAL_V1, STREAMS_V1

api_patterns = [
    re_path(
        r"", include(("haupt.apis.versions.urls", "versions"), namespace="versions")
    ),
]

api_patterns += [
    re_path(
        r"",
        include(
            ("haupt.apis.project_resources.urls", "project_resources"),
            namespace="project_resources",
        ),
    ),
    re_path(
        r"",
        include(
            ("haupt.apis.run_lineage.urls", "run_lineage"), namespace="run_lineage"
        ),
    ),
    re_path(r"", include(("haupt.apis.runs.urls", "runs"), namespace="runs")),
    re_path(
        r"", include(("haupt.apis.projects.urls", "projects"), namespace="projects")
    ),
]


# UI
projects_urls = "{}/{}".format(OWNER_NAME_PATTERN, PROJECT_NAME_PATTERN)
orgs_urls = "orgs/{}".format(OWNER_NAME_PATTERN)
ui_urlpatterns = [
    r"^$",
    r"^{}/?".format(orgs_urls),
    r"^{}/projects/?".format(orgs_urls),
    r"^{}/?$".format(projects_urls),
    r"^{}/runs.*/?".format(projects_urls),
    r"^{}/jobs.*/?".format(projects_urls),
    r"^{}/services.*/?".format(projects_urls),
    r"^{}/cli.*/?".format(projects_urls),
    r"^{}/settings/?".format(projects_urls),
    r"^{}/new.*/?".format(projects_urls),
]

# Streams
streams_routes = (
    logs_routes + k8s_routes + notifications_routes + artifacts_routes + events_routes
)
app_urlpatterns = [
    re_path(
        r"^{}/".format(INTERNAL_V1),
        include((internal_logs_routes, "internal-v1"), namespace="internal-v1"),
    ),
    re_path(
        r"^{}/".format(AUTH_REQUEST_V1),
        include((auth_request_routes, "auth-request-v1"), namespace="auth-request-v1"),
    ),
    re_path(
        r"^{}/".format(STREAMS_V1),
        include((streams_routes, "streams-v1"), namespace="streams-v1"),
    ),
    re_path(
        r"^{}/".format(API_V1), include((api_patterns, "api-v1"), namespace="api-v1")
    ),
]

handler404 = handler404
handler403 = handler403
handler500 = handler500
urlpatterns = get_urlpatterns(
    app_patterns=app_urlpatterns, no_healthz=False, ui_urlpatterns=ui_urlpatterns
)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

UUID_KEY = "uuid"
OWNER_KEY = "uuid"
NAME_KEY = "name"
USERNAME_KEY = "username"
VERSION_KEY = "version"
VERSION_PATTERN = r"(?P<version>[-\w]{1,16})"
INSTALLATION_KEY = "installation"
INSTALLATION_PATTERN = r"(?P<installation>\w{1,36})"
UUID_PATTERN = r"(?P<uuid>\w{1,36})"
SLUG_PATTERN = r"[-\w]{1,256}"
USERNAME_PATTERN = r"(?P<username>[-\w]{1,128})"
OWNER_NAME_KEY = "owner_name"
PROJECT_OWNER_NAME_KEY = "project_owner_name"
OWNER_NAME_PATTERN = r"(?P<owner_name>[-\w]{1,128})"
PROJECT_NAME_KEY = "project_name"
PROJECT_NAME_PATTERN = r"(?P<project_name>[-\w]{1,128})"
ARTIFACT_NAME_PATTERN = r"(?P<artifact_name>[-\w]{1,128})"
ARTIFACT_NAME_KEY = "artifact_name"
NAME_PATTERN = r"(?P<name>[-\w]{1,256})"
SEQUENCE_PATTERN = r"(?P<sequence>\d+)"
ID_PATTERN = r"(?P<id>\d+)"
INDEX_PATTERN = r"(?P<index>\d+)"
RUN_ID_PATTERN = r"(?P<run_id>\d+)"
RUN_UUID_KEY = "run_uuid"
RUN_UUID_PATTERN = r"(?P<run_uuid>\w{1,36})"

EVENT_KIND_PATTERN = r"(?P<event_kind>[-\w]{1,20})"
EVENT_NAME_PATTERN = r"(?P<event_name>[-\w]{1,128})"

ANY_CONSUMER_PATTERN = r"?(?P<path>.*)/?"



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.
import logging

from typing import Dict, Optional

from clipped.utils.json import orjson_loads
from rest_framework import status

from django.core.handlers.asgi import ASGIRequest
from django.db import transaction
from django.http import HttpResponse
from django.urls import path

from haupt.common.endpoints.validation import validate_methods
from haupt.streams.endpoints.base import UJSONResponse
from haupt.streams.tasks.notification import notify_run
from polyaxon import settings
from polyaxon.lifecycle import V1StatusCondition

logger = logging.getLogger("haupt.streams.notifications")


@transaction.non_atomic_requests
async def notify(
    request: ASGIRequest,
    namespace: str,
    owner: str,
    project: str,
    run_uuid: str,
    methods: Optional[Dict] = None,
) -> HttpResponse:
    validate_methods(request, methods)
    body = orjson_loads(request.body)
    run_name = body.get("name")
    condition = body.get("condition")
    if not condition:
        errors = "Received a notification request without condition."
        logger.warning(errors)
        return UJSONResponse(
            data={"errors": errors},
            status=status.HTTP_400_BAD_REQUEST,
        )
    condition = V1StatusCondition.get_condition(**condition)
    connections = body.get("connections")
    if not connections:
        errors = "Received a notification request without connections."
        logger.warning(errors)
        return UJSONResponse(
            data={"errors": errors},
            status=status.HTTP_400_BAD_REQUEST,
        )

    if not settings.AGENT_CONFIG.connections:
        errors = "Received a notification request, but the agent did not declare connections."
        logger.warning(errors)
        return UJSONResponse(
            data={"errors": errors},
            status=status.HTTP_400_BAD_REQUEST,
        )

    await notify_run(
        namespace=namespace,
        owner=owner,
        project=project,
        run_uuid=run_uuid,
        run_name=run_name,
        condition=condition,
        connections=connections,
    )
    return HttpResponse(status=status.HTTP_200_OK)


URLS_RUNS_NOTIFY = (
    "<str:namespace>/<str:owner>/<str:project>/runs/<str:run_uuid>/notify"
)

# fmt: off
notifications_routes = [
    path(
        URLS_RUNS_NOTIFY,
        notify,
        name="notify",
        kwargs=dict(methods=["POST"]),
    ),
]



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt.proxies.generators.api import generate_api_conf
from haupt.proxies.generators.forward import generate_forward_proxy_cmd
from haupt.proxies.generators.gateway import generate_gateway_conf
from haupt.proxies.generators.streams import generate_streams_conf



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from rest_framework import status
from rest_framework.response import Response

from haupt.apis.serializers.base.tags import TagsMixin
from haupt.common import auditor
from haupt.common.events.registry.run import (
    RUN_APPROVED_ACTOR,
    RUN_DELETED_ACTOR,
    RUN_STOPPED_ACTOR,
)
from haupt.db.abstracts.getter import get_run_model
from haupt.db.managers.statuses import bulk_new_run_status
from polyaxon import live_state
from polyaxon.lifecycle import LifeCycle, V1StatusCondition, V1Statuses
from polyaxon.schemas import V1RunPending


def create_runs_tags(view, request, *args, **kwargs):
    uuids = request.data.get("uuids", [])
    tags = request.data.get("tags", [])
    if not tags:
        return Response(status=status.HTTP_200_OK, data={})

    updated = []
    run_model = get_run_model()
    queryset = view.enrich_queryset(run_model.all)
    queryset = queryset.filter(uuid__in=uuids)
    for run in queryset.only("id", "tags"):
        run.tags = TagsMixin.validated_tags({"tags": tags, "merge": True}, run.tags)[
            "tags"
        ]
        updated.append(run)

        run_model.objects.bulk_update(updated, ["tags"])
    return Response(status=status.HTTP_200_OK, data={})


def stop_runs(view, request, actor, *args, **kwargs):
    uuids = request.data.get("uuids", [])
    # Immediate stop
    queryset = view.enrich_queryset(get_run_model().restorable)
    queryset = queryset.filter(uuid__in=uuids)
    queryset = queryset.filter(status__in=LifeCycle.SAFE_STOP_VALUES)
    condition = V1StatusCondition.get_condition(
        type=V1Statuses.STOPPED,
        status="True",
        reason="EventHandler",
        message="User requested to stop the run.",
    )
    bulk_new_run_status(queryset, condition)

    queryset = view.enrich_queryset(get_run_model().restorable)
    queryset = queryset.filter(uuid__in=uuids)
    queryset = queryset.exclude(status__in=LifeCycle.DONE_OR_IN_PROGRESS_VALUES)
    runs = [r for r in queryset]
    condition = V1StatusCondition.get_condition(
        type=V1Statuses.STOPPING,
        status="True",
        reason="EventHandler",
        message="User requested to stop the run.",
    )
    bulk_new_run_status(runs, condition)
    # For Audit
    view.set_owner()
    for run in runs:
        auditor.record(
            event_type=RUN_STOPPED_ACTOR,
            instance=run,
            actor_id=actor.id,
            actor_name=actor.username,
            owner_id=view._owner_id,
            owner_name=view.owner_name,
            project_name=view.project_name,
        )

    return Response(status=status.HTTP_200_OK, data={})


def approve_runs(view, request, actor, *args, **kwargs):
    uuids = request.data.get("uuids", [])
    queryset = view.enrich_queryset(get_run_model().objects)
    queryset = queryset.filter(uuid__in=uuids)
    queryset = queryset.filter(
        pending__in={V1RunPending.APPROVAL, V1RunPending.CACHE},
    )
    runs = [r for r in queryset]
    queryset.update(pending=None)
    # For Audit
    view.set_owner()
    for run in runs:
        auditor.record(
            event_type=RUN_APPROVED_ACTOR,
            instance=run,
            actor_id=actor.id,
            actor_name=actor.username,
            owner_id=view._owner_id,
            owner_name=view.owner_name,
            project_name=view.project_name,
        )

    return Response(status=status.HTTP_200_OK, data={})


def delete_runs(view, request, actor, *args, **kwargs):
    uuids = request.data.get("uuids", [])
    queryset = view.enrich_queryset(get_run_model().restorable)
    runs = queryset.filter(uuid__in=uuids)
    # Delete non managed immediately
    runs.filter(is_managed=False).delete()
    # For Audit
    view.set_owner()
    for run in runs:
        auditor.record(
            event_type=RUN_DELETED_ACTOR,
            instance=run,
            actor_id=actor.id,
            actor_name=actor.username,
            owner_id=view._owner_id,
            owner_name=view.owner_name,
            project_name=view.project_name,
        )
    # Deletion in progress
    runs.filter(is_managed=True).update(
        live_state=live_state.STATE_DELETION_PROGRESSING
    )
    return Response(status=status.HTTP_200_OK, data={})



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from django.apps import apps as django_apps
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


def get_db_model_name(name: str) -> str:
    db = settings.AUTH_USER_MODEL.split(".")[0]
    return "{}.{}".format(db, name)


def get_db_model(name: str) -> str:
    model_name = get_db_model_name(name)
    try:
        return django_apps.get_model(model_name, require_ready=False)
    except ValueError:
        raise ImproperlyConfigured(
            "{} must be of the form 'app_label.model_name'".format(model_name)
        )
    except LookupError:
        raise ImproperlyConfigured(
            "{} refers to model '{}' that has not been installed".format(
                name, model_name
            )
        )


def get_project_model():
    return get_db_model("Project")


def get_run_model():
    return get_db_model("Run")


def get_artifact_model():
    return get_db_model("Artifact")


def get_lineage_model():
    return get_db_model("ArtifactLineage")



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt import settings
from haupt.proxies.schemas.api.uwsgi import get_uwsgi_config
from haupt.proxies.schemas.base import clean_config
from haupt.proxies.schemas.locations import get_api_locations_config
from haupt.proxies.schemas.scaffold import get_scaffold_config


def get_base_config():
    api_configs = [
        get_uwsgi_config(),
    ]
    api_location_configs = [get_api_locations_config()]
    config = get_scaffold_config(
        is_proxy=False,
        port=settings.PROXIES_CONFIG.api_target_port,
        use_ssl_config=False,
        use_assets_config=True,
        use_services_configs=False,
        api_configs=api_configs,
        api_location_configs=api_location_configs,
    )

    return clean_config(config)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

import uuid

from django.core.validators import validate_slug
from django.db import models

from haupt.common.validation.blacklist import validate_blacklist_name
from haupt.db.abstracts.catalogs import BaseLiveStateCatalog
from haupt.db.abstracts.readme import ReadmeModel
from polyaxon.constants.globals import DEFAULT


class Owner:
    name = DEFAULT
    uuid = uuid.UUID("9b0a3806e3f84ea1959a7842e34129ed")
    id = 1


class Actor:
    username = DEFAULT
    id = 1


class BaseProject(BaseLiveStateCatalog, ReadmeModel):
    name = models.CharField(
        max_length=128, validators=[validate_slug, validate_blacklist_name], unique=True
    )

    class Meta(BaseLiveStateCatalog.Meta):
        abstract = True

    @property
    def owner(self):
        return Owner

    @property
    def owner_id(self):
        return Owner.id

    @property
    def actor(self):
        return Actor
